/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.activities;

import android.content.Intent;
import android.os.Bundle;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.adapter.WorklistFullTestItemAdapter;
import com.uoc.tfm.analyzerhandler.database.WorklistDAO;
import com.uoc.tfm.analyzerhandler.entities.AnalyzerData;
import com.uoc.tfm.analyzerhandler.entities.WorklistTestData;

import java.util.ArrayList;

public class AnalyzerFullWorklistActivity extends AppCompatActivity {

    private SwipeRefreshLayout swipeRefreshLayout;

    private WorklistFullTestItemAdapter mAnalyzerItemAdapter;
    private WorklistDAO worklistDAO;
    private AnalyzerData analyzerData;
    private ArrayList<WorklistTestData> worklistTestDataArrayList;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        analyzerData = (AnalyzerData) getIntent().getSerializableExtra("Analyzer");
        setTitle(analyzerData.getCode() + " - Full Worklist");

        setContentView(R.layout.full_worklist_activity);


        TextView analyzerDescription = (TextView) findViewById(R.id.full_worklist_analyzer_description);
        TextView info = (TextView) findViewById(R.id.full_worklist_analyzer_info);
        TextView warning = (TextView) findViewById(R.id.full_worklist_analyzer_warning);
        TextView alarms = (TextView) findViewById(R.id.full_worklist_analyzer_alarm);
        TextView status = (TextView) findViewById(R.id.full_worklist_analyzer_status);
        ImageView connected = (ImageView) findViewById(R.id.full_worklist_connected);

        analyzerDescription.setText(analyzerData.getDescription());
        info.setText(analyzerData.getInfo() + "");
        warning.setText(analyzerData.getWarnings() + "");
        alarms.setText(analyzerData.getAlarms() + "");

        info.setVisibility(analyzerData.getInfo() > 0 ? View.VISIBLE : View.GONE);
        warning.setVisibility(analyzerData.getWarnings() > 0 ? View.VISIBLE : View.GONE);
        alarms.setVisibility(analyzerData.getAlarms() > 0 ? View.VISIBLE : View.GONE);

        connected.setVisibility(analyzerData.getConnected() ? View.GONE : View.VISIBLE);

        if (analyzerData.getConnected()) {
            if (analyzerData.getAlarms() > 0) {
                status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_red_dark));
            } else if (analyzerData.getWarnings() > 0) {
                status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_orange_light));
            } else {
                status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_green_light));
            }
        } else {
            status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.darker_gray));
        }
        swipeRefreshLayout = (SwipeRefreshLayout) findViewById(R.id.swipeLayoutWorklist);
        swipeRefreshLayout.setColorSchemeResources(
                R.color.colorAccent,
                R.color.lightBackground,
                R.color.hintColorLight);
        swipeRefreshLayout.setOnRefreshListener(
                new SwipeRefreshLayout.OnRefreshListener() {
                    @Override
                    public void onRefresh() {
                        worklistDAO = new WorklistDAO(analyzerData.getId(), worklistTestDataArrayList, mAnalyzerItemAdapter, swipeRefreshLayout);
                        worklistDAO.execute();
                    }
                });

        TextView emptyMessage = (TextView) findViewById(R.id.full_worklist_analyzer_empty_view);

        worklistTestDataArrayList = (analyzerData.getWorklist() != null && analyzerData.getWorklist().size() > 0) ? analyzerData.getWorklist().get(0).getWorklistTestList() : new ArrayList<WorklistTestData>();
        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        mAnalyzerItemAdapter = new WorklistFullTestItemAdapter(worklistTestDataArrayList);
        mAnalyzerItemAdapter.setEmptyView(emptyMessage);

        assert recyclerView != null;
        recyclerView.setAdapter(mAnalyzerItemAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.full_worklist_detail_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {

            case R.id.action_test_configuration_full_worklist:
                Intent intent = new Intent(AnalyzerFullWorklistActivity.this, AnalyzerTestConfigurationActivity.class);
                intent.putExtra("Analyzer", analyzerData);
                startActivity(intent);
                return true;

            default:
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                return super.onOptionsItemSelected(item);

        }
    }
}
