/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.activities;

import android.os.Bundle;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.adapter.AnalyzerTestConfigurationItemAdapter;
import com.uoc.tfm.analyzerhandler.database.TestsDAO;
import com.uoc.tfm.analyzerhandler.entities.AnalyzerData;
import com.uoc.tfm.analyzerhandler.entities.TestData;

import java.util.ArrayList;

public class AnalyzerTestConfigurationActivity extends AppCompatActivity {

    private SwipeRefreshLayout swipeRefreshLayout;

    private AnalyzerTestConfigurationItemAdapter mAnalyzerItemAdapter;
    private TestsDAO testsDAO;
    private AnalyzerData analyzerData;
    private ArrayList<TestData> testDataArrayList;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        analyzerData = (AnalyzerData) getIntent().getSerializableExtra("Analyzer");
        setTitle(analyzerData.getCode() + " - " + getApplicationContext().getString(R.string.menu_tests));

        setContentView(R.layout.analyzer_test_configuration_activity);


        TextView analyzerDescription = (TextView) findViewById(R.id.analyzer_test_analyzer_description);
        TextView info = (TextView) findViewById(R.id.analyzer_test_analyzer_info);
        TextView warning = (TextView) findViewById(R.id.analyzer_test_analyzer_warning);
        TextView alarms = (TextView) findViewById(R.id.analyzer_test_analyzer_alarm);
        TextView status = (TextView) findViewById(R.id.analyzer_test_analyzer_status);
        ImageView connected = (ImageView) findViewById(R.id.analyzer_test_analyzer_connected);

        analyzerDescription.setText(analyzerData.getDescription());
        info.setText(analyzerData.getInfo() + "");
        warning.setText(analyzerData.getWarnings() + "");
        alarms.setText(analyzerData.getAlarms() + "");

        info.setVisibility(analyzerData.getInfo() > 0 ? View.VISIBLE : View.GONE);
        warning.setVisibility(analyzerData.getWarnings() > 0 ? View.VISIBLE : View.GONE);
        alarms.setVisibility(analyzerData.getAlarms() > 0 ? View.VISIBLE : View.GONE);

        connected.setVisibility(analyzerData.getConnected() ? View.GONE : View.VISIBLE);
        if (analyzerData.getConnected()) {
            if (analyzerData.getAlarms() > 0) {
                status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_red_dark));
            } else if (analyzerData.getWarnings() > 0) {
                status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_orange_light));
            } else {
                status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_green_light));
            }
        } else {
            status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.darker_gray));
        }
        swipeRefreshLayout = (SwipeRefreshLayout) findViewById(R.id.swipeLayoutWorklist);
        swipeRefreshLayout.setColorSchemeResources(
                R.color.colorAccent,
                R.color.lightBackground,
                R.color.hintColorLight);
        swipeRefreshLayout.setOnRefreshListener(
                new SwipeRefreshLayout.OnRefreshListener() {
                    @Override
                    public void onRefresh() {
                        testsDAO = new TestsDAO(analyzerData.getId(), testDataArrayList, mAnalyzerItemAdapter, swipeRefreshLayout);
                        testsDAO.execute();
                    }
                });

        TextView emptyMessage = (TextView) findViewById(R.id.analyzer_test_empty_view);
        testDataArrayList = analyzerData.getConfiguredTests();
        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        mAnalyzerItemAdapter = new AnalyzerTestConfigurationItemAdapter(testDataArrayList);
        mAnalyzerItemAdapter.setEmptyView(emptyMessage);

        assert recyclerView != null;
        recyclerView.setAdapter(mAnalyzerItemAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

    }

}
