/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.activities;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.NavigationView;
import android.support.v4.app.NavUtils;
import android.support.v4.app.TaskStackBuilder;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.adapter.AnalyzerItemAdapter;
import com.uoc.tfm.analyzerhandler.adapter.RecyclerViewOnItemClickListener;
import com.uoc.tfm.analyzerhandler.common.AppGlobals;
import com.uoc.tfm.analyzerhandler.database.AnalyzersDAO;
import com.uoc.tfm.analyzerhandler.entities.AnalyzerData;
import com.uoc.tfm.analyzerhandler.entities.UserData;

import java.util.ArrayList;

public class MainActivity extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener {

    private SwipeRefreshLayout swipeRefreshLayout;

    private AnalyzerItemAdapter mAnalyzerItemAdapter;
    private ArrayList<AnalyzerData> analyzersItems = new ArrayList<>();
    private AnalyzersDAO analyzersDAO;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        assert drawer != null;
        drawer.addDrawerListener(toggle);

        toggle.syncState();

        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        assert navigationView != null;
        navigationView.setNavigationItemSelectedListener(this);

        swipeRefreshLayout = (SwipeRefreshLayout) findViewById(R.id.swipeLayout);
        swipeRefreshLayout.setColorSchemeResources(
                R.color.colorAccent,
                R.color.lightBackground,
                R.color.hintColorLight);
        swipeRefreshLayout.setOnRefreshListener(
                new SwipeRefreshLayout.OnRefreshListener() {
                    @Override
                    public void onRefresh() {
                        analyzersDAO = new AnalyzersDAO(analyzersItems, mAnalyzerItemAdapter, swipeRefreshLayout, AnalyzersDAO.GET_ALL);
                        analyzersDAO.execute();
                    }
                });

        TextView emptyMessage = (TextView) findViewById(R.id.list_analyzers_empty_view);

        RecyclerView recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        mAnalyzerItemAdapter = new AnalyzerItemAdapter(analyzersItems, new RecyclerViewOnItemClickListener() {
            @Override
            public void onClick(View v, int position) {
                AnalyzerData analyzerData = analyzersItems.get(position);

                Intent intent = new Intent(MainActivity.this, AnalyzerWorklistActivity.class);
                intent.putExtra("Analyzer", analyzerData);
                startActivity(intent);
            }
        });
        mAnalyzerItemAdapter.setEmptyView(emptyMessage);

        assert recyclerView != null;
        recyclerView.setAdapter(mAnalyzerItemAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        View navHeaderView = navigationView.getHeaderView(0);

        TextView textViewUsername = (TextView) navHeaderView.findViewById(R.id.nav_username_txt);
        TextView textViewUserEmail = (TextView) navHeaderView.findViewById(R.id.nav_user_email);

        assert textViewUsername != null;
        textViewUsername.setText(AppGlobals.userData.getUsername());

        assert textViewUserEmail != null;
        textViewUserEmail.setText(AppGlobals.userData.getEmail());


        analyzersDAO = new AnalyzersDAO(analyzersItems, mAnalyzerItemAdapter, swipeRefreshLayout, AnalyzersDAO.GET_ALL);
        analyzersDAO.execute();
    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        assert drawer != null;
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        // Handle navigation view item clicks here.
        int id = item.getItemId();

        if (id == R.id.nav_notification_preferences) {

            Intent intent = new Intent(MainActivity.this, NotificationSettingsActivity.class);
            startActivity(intent);

        } else if (id == R.id.nav_logout) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle(R.string.alert_logout_title) //
                    .setMessage(R.string.alert_logout_message) //
                    .setPositiveButton(getString(R.string.accept), new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            dialog.dismiss();
                            AppGlobals.userData = new UserData();

                            Intent upIntent = NavUtils.getParentActivityIntent(MainActivity.this);
                            if (NavUtils.shouldUpRecreateTask(MainActivity.this, upIntent)) {
                                // This activity is NOT part of this app's task, so create a new task
                                // when navigating up, with a synthesized back stack.
                                TaskStackBuilder.create(MainActivity.this)
                                        // Add all of this activity's parents to the back stack
                                        .addNextIntentWithParentStack(upIntent)
                                        // Navigate up to the closest parent
                                        .startActivities();
                            } else {
                                // This activity is part of this app's task, so simply
                                // navigate up to the logical parent activity.
                                NavUtils.navigateUpTo(MainActivity.this, upIntent);
                            }
                        }
                    }) //
                    .setNegativeButton(getString(R.string.decline), new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            dialog.dismiss();
                        }
                    });
            builder.show();
        }

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        assert drawer != null;
        drawer.closeDrawer(GravityCompat.START);
        return true;
    }
}
