/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.activities;


import android.annotation.TargetApi;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Build;
import android.os.Bundle;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceFragment;
import android.preference.PreferenceManager;
import android.support.v4.app.NavUtils;
import android.support.v4.app.TaskStackBuilder;
import android.support.v7.app.ActionBar;
import android.view.MenuItem;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.common.PushNotificationUtil;

import java.util.ArrayList;

public class NotificationSettingsActivity extends AppCompatPreferenceActivity {

    private static boolean alarms = false;
    private static boolean warnings = false;
    private static boolean info = false;
    private static boolean connection = false;

    private static Preference.OnPreferenceChangeListener sBindPreferenceSummaryToValueListener = new Preference.OnPreferenceChangeListener() {

        // TODO: save to the database
        @Override
        public boolean onPreferenceChange(Preference preference, Object value) {
            String stringValue = value.toString();

            if (preference instanceof ListPreference) {
                // For list preferences, look up the correct display value in
                // the preference's 'entries' list.
                ListPreference listPreference = (ListPreference) preference;
                int index = listPreference.findIndexOfValue(stringValue);

                // Set the summary to reflect the new value.
                preference.setSummary(
                        index >= 0
                                ? listPreference.getEntries()[index]
                                : null);
            } else {
                // For all other preferences, set the summary to the value's
                // simple string representation.
                enableNotification("true".equals(stringValue), getCategory(preference.getKey()));

                preference.setSummary(stringValue);

            }
            return true;
        }
    };

    private static void enableNotification(boolean pEnable, String pCategory) {
        switch (pCategory) {
            case "A":
                alarms = pEnable;
                break;
            case "W":
                warnings = pEnable;
                break;
            case "I":
                info = pEnable;
                break;
            case "C":
                connection = pEnable;
                break;
        }
    }

    private static String getCategory(String pPreferenceKey) {
        switch (pPreferenceKey) {
            case "pref_analyzer_alarms":
                return "A";
            case "pref_analyzer_warnings":
                return "W";
            case "pref_analyzer_connection":
                return "C";
            case "pref_test_warnings":
                return "I";
        }
        return "A";
    }

    /**
     * Helper method to determine if the device has an extra-large screen. For
     * example, 10" tablets are extra-large.
     */
    private static boolean isXLargeTablet(Context context) {
        return (context.getResources().getConfiguration().screenLayout
                & Configuration.SCREENLAYOUT_SIZE_MASK) >= Configuration.SCREENLAYOUT_SIZE_XLARGE;
    }

    /**
     * Binds a preference's summary to its value. More specifically, when the
     * preference's value is changed, its summary (line of text below the
     * preference title) is updated to reflect the value. The summary is also
     * immediately updated upon calling this method. The exact display format is
     * dependent on the type of preference.
     *
     * @see #sBindPreferenceSummaryToValueListener
     */
    private static void bindPreferenceSummaryToValue(Preference preference) {
        // Set the listener to watch for value changes.
        preference.setOnPreferenceChangeListener(sBindPreferenceSummaryToValueListener);

        // Trigger the listener immediately with the preference's
        // current value.
        sBindPreferenceSummaryToValueListener.onPreferenceChange(preference,
                PreferenceManager
                        .getDefaultSharedPreferences(preference.getContext())
                        .getBoolean(preference.getKey(), true));
        enableNotification(PreferenceManager
                .getDefaultSharedPreferences(preference.getContext())
                .getBoolean(preference.getKey(), true), preference.getKey());
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setupActionBar();
        getFragmentManager().beginTransaction().replace(android.R.id.content,
                new NotificationPreferenceFragment()).commit();

    }

    /**
     * Set up the {@link android.app.ActionBar}, if the API is available.
     */
    private void setupActionBar() {
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            // Show the Up button in the action bar.
            actionBar.setDisplayHomeAsUpEnabled(true);
            actionBar.setTitle(getText(R.string.notification_title));
        }
    }

    @Override
    public boolean onMenuItemSelected(int featureId, MenuItem item) {
        Intent upIntent = NavUtils.getParentActivityIntent(NotificationSettingsActivity.this);
        if (NavUtils.shouldUpRecreateTask(NotificationSettingsActivity.this, upIntent)) {
            // This activity is NOT part of this app's task, so create a new task
            // when navigating up, with a synthesized back stack.
            TaskStackBuilder.create(NotificationSettingsActivity.this)
                    // Add all of this activity's parents to the back stack
                    .addNextIntentWithParentStack(upIntent)
                    // Navigate up to the closest parent
                    .startActivities();
        } else {
            // This activity is part of this app's task, so simply
            // navigate up to the logical parent activity.
            NavUtils.navigateUpTo(NotificationSettingsActivity.this, upIntent);
        }

        saveNotificationPreferences();
        return true;
    }

    private void saveNotificationPreferences() {
        ArrayList<String> categoriesList = new ArrayList<>();
        if (alarms) {
            categoriesList.add("A");
        }
        if (warnings) {
            categoriesList.add("W");
        }
        if (info) {
            categoriesList.add("I");
        }
        if (connection) {
            categoriesList.add("C");
        }
        PushNotificationUtil.registerToANotification(getApplicationContext(), categoriesList);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean onIsMultiPane() {
        return isXLargeTablet(this);
    }


    /**
     * This method stops fragment injection in malicious applications.
     * Make sure to deny any unknown fragments here.
     */
    protected boolean isValidFragment(String fragmentName) {
        return PreferenceFragment.class.getName().equals(fragmentName)
                || NotificationPreferenceFragment.class.getName().equals(fragmentName);
    }

    /**
     * This fragment shows notification preferences only. It is used when the
     * activity is showing a two-pane settings UI.
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public static class NotificationPreferenceFragment extends PreferenceFragment {
        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            addPreferencesFromResource(R.xml.pref_notification);
            setHasOptionsMenu(true);

            // Bind the summaries of EditText/List/Dialog/Ringtone preferences
            // to their values. When their values change, their summaries are
            // updated to reflect the new value, per the Android Design
            // guidelines.
            bindPreferenceSummaryToValue(findPreference("pref_analyzer_alarms"));
            bindPreferenceSummaryToValue(findPreference("pref_analyzer_warnings"));
            bindPreferenceSummaryToValue(findPreference("pref_analyzer_connection"));
            bindPreferenceSummaryToValue(findPreference("pref_test_warnings"));
        }

        @Override
        public boolean onOptionsItemSelected(MenuItem item) {
            int id = item.getItemId();
            if (id == android.R.id.home) {
                startActivity(new Intent(getActivity(), NotificationSettingsActivity.class));
                return true;
            }
            return super.onOptionsItemSelected(item);
        }
    }
}
