/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.activities;

import android.os.Bundle;
import android.support.design.widget.TabLayout;
import android.support.v4.content.ContextCompat;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.adapter.NotificationsViewPagerAdapter;
import com.uoc.tfm.analyzerhandler.entities.AnalyzerData;

public class NotificationsAnalyzerActivity extends AppCompatActivity {
    private AnalyzerData analyzerData;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.notifications_main);

        analyzerData = (AnalyzerData) getIntent().getSerializableExtra("Analyzer");

        setTitle(analyzerData.getCode() + " - " + getApplicationContext().getString(R.string.common_notifications));

        TabLayout tabLayout = (TabLayout) findViewById(R.id.notifications_tabs);
        final ViewPager viewPager = (ViewPager) findViewById(R.id.notifications_viewpager);

        NotificationsViewPagerAdapter viewPagerAdapter = new NotificationsViewPagerAdapter(getSupportFragmentManager());
        viewPager.setAdapter(viewPagerAdapter);


        final TabLayout.Tab alarmsTab = tabLayout.newTab();
        final TabLayout.Tab warningsTab = tabLayout.newTab();
        final TabLayout.Tab infoTabs = tabLayout.newTab();

        TextView analyzerDescription = (TextView) findViewById(R.id.notifications_analyzer_description);
        TextView info = (TextView) findViewById(R.id.notifications_analyzer_info);
        TextView warning = (TextView) findViewById(R.id.notifications_analyzer_warning);
        TextView alarms = (TextView) findViewById(R.id.notifications_analyzer_alarm);
        TextView status = (TextView) findViewById(R.id.notifications_analyzers_status);
        ImageView connected = (ImageView) findViewById(R.id.notifications_analyzer_connected);

        analyzerDescription.setText(analyzerData.getDescription());
        info.setText(analyzerData.getInfo() + "");
        warning.setText(analyzerData.getWarnings() + "");
        alarms.setText(analyzerData.getAlarms() + "");

        info.setVisibility(analyzerData.getInfo() > 0 ? View.VISIBLE : View.GONE);
        warning.setVisibility(analyzerData.getWarnings() > 0 ? View.VISIBLE : View.GONE);
        alarms.setVisibility(analyzerData.getAlarms() > 0 ? View.VISIBLE : View.GONE);
        connected.setVisibility(analyzerData.getConnected() ? View.GONE : View.VISIBLE);

        if (analyzerData.getConnected()) {
            if (analyzerData.getAlarms() > 0) {
                status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_red_dark));
            } else if (analyzerData.getWarnings() > 0) {
                status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_orange_light));
            } else {
                status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_green_light));
            }
        } else {
            status.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), android.R.color.darker_gray));
        }
        alarmsTab.setText(getApplicationContext().getString(R.string.common_alarms));
        warningsTab.setText(getApplicationContext().getString(R.string.common_warnings));
        infoTabs.setText(getApplicationContext().getString(R.string.common_info));

        tabLayout.addTab(infoTabs, 0);
        tabLayout.addTab(warningsTab, 1);
        tabLayout.addTab(alarmsTab, 2);

        tabLayout.setTabTextColors(ContextCompat.getColorStateList(this, R.color.colorPrimary));
        tabLayout.setSelectedTabIndicatorColor(ContextCompat.getColor(this, R.color.colorAccent));

        viewPager.addOnPageChangeListener(new TabLayout.TabLayoutOnPageChangeListener(tabLayout));
        tabLayout.setOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {
            @Override
            public void onTabSelected(TabLayout.Tab tab) {
                viewPager.setCurrentItem(tab.getPosition(), true);
            }

            @Override
            public void onTabUnselected(TabLayout.Tab tab) {

            }

            @Override
            public void onTabReselected(TabLayout.Tab tab) {

            }
        });
    }

    public AnalyzerData getAnalyzerData() {
        return analyzerData;
    }
}
