/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.adapter;

import android.support.v4.content.ContextCompat;
import android.support.v7.widget.RecyclerView;
import android.util.Pair;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.entities.AnalyzerData;
import com.uoc.tfm.analyzerhandler.entities.WorklistTestData;

import java.util.ArrayList;

/**
 * @author Ruben Carmona
 * @project TFM - Analyzer Handler
 * @date 05/2016
 */
public class AnalyzerItemAdapter extends RecyclerView.Adapter<AnalyzerItemAdapter.AnalyzerViewHolder> {//ArrayAdapter<AnalyzerData> {

    private TextView emptyView;
    private ArrayList<AnalyzerData> analyzersItems;

    final RecyclerView.AdapterDataObserver observer = new RecyclerView.AdapterDataObserver() {
        @Override
        public void onChanged() {
            super.onChanged();
            checkIfEmpty();
        }
    };
    private RecyclerViewOnItemClickListener recyclerViewOnItemClickListener;

    public AnalyzerItemAdapter(ArrayList<AnalyzerData> pAnalyzerItems, RecyclerViewOnItemClickListener pRecyclerViewOnItemClickListener) {
        this.analyzersItems = pAnalyzerItems;
        this.recyclerViewOnItemClickListener = pRecyclerViewOnItemClickListener;
        this.registerAdapterDataObserver(observer);
    }

    @Override
    public AnalyzerViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View row = LayoutInflater.from(parent.getContext()).inflate(R.layout.list_analyzers_item, parent, false);
        AnalyzerViewHolder pvh = new AnalyzerViewHolder(row, recyclerViewOnItemClickListener);
        return pvh;
    }

    @Override
    public void onBindViewHolder(AnalyzerViewHolder holder, int position) {
        AnalyzerData aAnalyzerData = analyzersItems.get(position);

        holder.mAnalyzerDescription.setText(aAnalyzerData.getDescription());

        Pair<WorklistTestData, WorklistTestData> currentAndNextTests = aAnalyzerData.getCurrentTestFromWorklist();

        if (currentAndNextTests.first != null) {

            holder.mCurrentTest.setText(currentAndNextTests.first.getDescription());
            holder.mMinutesLeft.setText(currentAndNextTests.first.getMinutes() + " min");
        } else {
            holder.mCurrentTest.setText("--");
            holder.mMinutesLeft.setText("-- min");
        }

        if (currentAndNextTests.second != null) {
            holder.mNextTest.setText(holder.getmAnalyzerStatus().getContext().getString(R.string.common_next) + "\n" + currentAndNextTests.second.getDescription());

        } else {
            holder.mNextTest.setText(holder.getmAnalyzerStatus().getContext().getString(R.string.common_next) + "\n--");
        }

        holder.mInfo.setText(Integer.toString(aAnalyzerData.getInfo()));
        holder.mWarning.setText(Integer.toString(aAnalyzerData.getWarnings()));
        holder.mAlarms.setText(Integer.toString(aAnalyzerData.getAlarms()));

        holder.mInfo.setVisibility(aAnalyzerData.getInfo() > 0 ? View.VISIBLE : View.GONE);
        holder.mWarning.setVisibility(aAnalyzerData.getWarnings() > 0 ? View.VISIBLE : View.GONE);
        holder.mAlarms.setVisibility(aAnalyzerData.getAlarms() > 0 ? View.VISIBLE : View.GONE);

        if (aAnalyzerData.getConnected()) {
            holder.mOverlayConnection.setVisibility(View.GONE);
            holder.mConnectionIcon.setVisibility(View.GONE);
            if (aAnalyzerData.getAlarms() > 0) {
                holder.mAnalyzerStatus.setBackgroundColor(ContextCompat.getColor(holder.getmAnalyzerStatus().getContext(), android.R.color.holo_red_dark));
            } else if (aAnalyzerData.getWarnings() > 0) {
                holder.mAnalyzerStatus.setBackgroundColor(ContextCompat.getColor(holder.getmAnalyzerStatus().getContext(), android.R.color.holo_orange_light));
            } else {
                holder.mAnalyzerStatus.setBackgroundColor(ContextCompat.getColor(holder.getmAnalyzerStatus().getContext(), android.R.color.holo_green_light));
            }
        } else {
            holder.mOverlayConnection.setVisibility(View.VISIBLE);
            holder.mConnectionIcon.setVisibility(View.VISIBLE);
            holder.mAnalyzerStatus.setBackgroundColor(ContextCompat.getColor(holder.getmAnalyzerStatus().getContext(), android.R.color.darker_gray));
        }
    }

    public void setEmptyView(TextView pEmptyView) {
        this.emptyView = pEmptyView;
    }

    private void checkIfEmpty() {
        if (emptyView != null) {
            emptyView.setVisibility(analyzersItems.size() > 0 ? View.GONE : View.VISIBLE);
        }
    }

    @Override
    public int getItemCount() {
        return analyzersItems.size();
    }

    static class AnalyzerViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
        private TextView mAnalyzerDescription;
        private TextView mCurrentTest;
        private TextView mInfo;
        private TextView mWarning;
        private TextView mAlarms;
        private TextView mMinutesLeft;
        private TextView mNextTest;
        private TextView mAnalyzerStatus;
        private ImageView mConnectionIcon;
        private TextView mOverlayConnection;

        private RecyclerViewOnItemClickListener recyclerViewOnItemClickListener;

        public AnalyzerViewHolder(View itemView, RecyclerViewOnItemClickListener recyclerViewOnItemClickListener) {
            super(itemView);
            mAnalyzerDescription = (TextView) itemView.findViewById(R.id.detail_analyzer_description);
            mCurrentTest = (TextView) itemView.findViewById(R.id.list_anaylzers_tests_description);
            mInfo = (TextView) itemView.findViewById(R.id.list_analyzers_info);
            mWarning = (TextView) itemView.findViewById(R.id.list_analyzers_warnings);
            mAlarms = (TextView) itemView.findViewById(R.id.list_analyzers_alarms);
            mMinutesLeft = (TextView) itemView.findViewById(R.id.worklist_test_start_time);
            mNextTest = (TextView) itemView.findViewById(R.id.list_analyzers_next_test);
            mAnalyzerStatus = (TextView) itemView.findViewById(R.id.list_analyzers_status);

            mConnectionIcon = (ImageView) itemView.findViewById(R.id.icon_connected);
            mOverlayConnection = (TextView) itemView.findViewById(R.id.overlay_layout);

            this.recyclerViewOnItemClickListener = recyclerViewOnItemClickListener;
            itemView.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {
            recyclerViewOnItemClickListener.onClick(v, getAdapterPosition());
        }

        public TextView getmAnalyzerStatus() {
            return mAnalyzerStatus;
        }

    }
}