/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or 
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any 
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by 
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.adapter;

import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.entities.TestData;

import java.util.ArrayList;

/**
 * @author Ruben Carmona
 * @project TFM - Analyzer Handler
 * @date 05/2016
 */
public class AnalyzerTestConfigurationItemAdapter extends RecyclerView.Adapter<AnalyzerTestConfigurationItemAdapter.AnalyzerTestViewHolder> {

    private TextView emptyView;
    private ArrayList<TestData> testDataList;

    final RecyclerView.AdapterDataObserver observer = new RecyclerView.AdapterDataObserver() {
        @Override
        public void onChanged() {
            super.onChanged();
            checkIfEmpty();
        }
    };

    public AnalyzerTestConfigurationItemAdapter(ArrayList<TestData> pTestDataList) {

        this.testDataList = pTestDataList;

        this.registerAdapterDataObserver(observer);
    }

    @Override
    public AnalyzerTestViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View row = LayoutInflater.from(parent.getContext()).inflate(R.layout.analyzer_test_configuration_detail_item, parent, false);
        AnalyzerTestViewHolder pvh = new AnalyzerTestViewHolder(row);
        return pvh;
    }

    @Override
    public void onBindViewHolder(AnalyzerTestViewHolder holder, int position) {
        TestData aTestData = testDataList.get(position);

        holder.mTestCode.setText(aTestData.getCode());
        holder.mTestDescription.setText(aTestData.getDescription());
        holder.mMinutes.setText(aTestData.getMinutes().toString() + " min");
        holder.mRemainingTests.setText(aTestData.getFeasibility() + "");
        holder.mRemainingFeasibility.setText(aTestData.getFeasibilityWarning() + "");
        holder.mChecked.setBackground(holder.getmTestCode().getResources().getDrawable(aTestData.getFeasibility() > aTestData.getFeasibilityWarning() ? R.drawable.ic_checked : R.drawable.ic_cancel));
    }

    public void setEmptyView(TextView pEmptyView) {
        this.emptyView = pEmptyView;
    }

    private void checkIfEmpty() {
        if (emptyView != null) {
            emptyView.setVisibility(testDataList.size() > 0 ? View.GONE : View.VISIBLE);
        }
    }

    @Override
    public int getItemCount() {
        return testDataList.size();
    }

    static class AnalyzerTestViewHolder extends RecyclerView.ViewHolder {
        private TextView mTestCode;
        private TextView mTestDescription;
        private TextView mRemainingTests;
        private TextView mMinutes;
        private TextView mRemainingFeasibility;
        private ImageView mChecked;

        public AnalyzerTestViewHolder(View itemView) {
            super(itemView);
            mTestCode = (TextView) itemView.findViewById(R.id.test_configuration_code);
            mTestDescription = (TextView) itemView.findViewById(R.id.test_configuration_description);
            mRemainingTests = (TextView) itemView.findViewById(R.id.test_configuration_number_of_tests);
            mMinutes = (TextView) itemView.findViewById(R.id.test_configuration_minutes);
            mRemainingFeasibility = (TextView) itemView.findViewById(R.id.test_configuration_feasibility);
            mChecked = (ImageView) itemView.findViewById(R.id.test_configuration_checked);
        }


        public TextView getmTestCode() {
            return mTestCode;
        }

        public TextView getmTestDescription() {
            return mTestDescription;
        }

        public TextView getmRemainingTests() {
            return mRemainingTests;
        }

        public TextView getmMinutes() {
            return mMinutes;
        }

        public TextView getmRemainingFeasibility() {
            return mRemainingFeasibility;
        }

    }
}