/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.adapter;

import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.entities.NotificationData;

import java.text.DateFormat;
import java.util.ArrayList;

/**
 * @author Ruben Carmona
 * @project TFM - Analyzer Handler
 * @date 05/2016
 */
public class NotificationsItemAdapter extends RecyclerView.Adapter<NotificationsItemAdapter.NotificationViewHolder> {

    private TextView emptyView;
    private ArrayList<NotificationData> notificationArrayList;

    final RecyclerView.AdapterDataObserver observer = new RecyclerView.AdapterDataObserver() {
        @Override
        public void onChanged() {
            super.onChanged();
            checkIfEmpty();
        }
    };

    public NotificationsItemAdapter(ArrayList<NotificationData> pNotificationArraylist) {

        this.notificationArrayList = pNotificationArraylist;

        this.registerAdapterDataObserver(observer);
    }

    @Override
    public NotificationViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View row = LayoutInflater.from(parent.getContext()).inflate(R.layout.list_notifications_item, parent, false);
        NotificationViewHolder pvh = new NotificationViewHolder(row);
        return pvh;
    }

    @Override
    public void onBindViewHolder(NotificationViewHolder holder, int position) {
        java.text.DateFormat dateFormat = DateFormat.getDateTimeInstance();

        NotificationData notificationData = notificationArrayList.get(position);

        holder.mNotificationDescription.setText(notificationData.getDescription());
        holder.mNotificationDate.setText(dateFormat.format(notificationData.getTime()));
    }

    public void setEmptyView(TextView pEmptyView) {
        this.emptyView = pEmptyView;
    }

    private void checkIfEmpty() {
        if (emptyView != null) {
            emptyView.setVisibility(notificationArrayList.size() > 0 ? View.GONE : View.VISIBLE);
        }
    }

    @Override
    public int getItemCount() {
        return notificationArrayList.size();
    }

    static class NotificationViewHolder extends RecyclerView.ViewHolder {
        private TextView mNotificationDescription;
        private TextView mNotificationDate;

        public NotificationViewHolder(View itemView) {
            super(itemView);
            mNotificationDescription = (TextView) itemView.findViewById(R.id.list_notifications_alarms_description);
            mNotificationDate = (TextView) itemView.findViewById(R.id.list_notifications_alarms_date);
        }

        public TextView getmNotificationDate() {
            return mNotificationDate;
        }
    }

}
