/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or 
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any 
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by 
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.adapter;

import android.support.v7.widget.RecyclerView;
import android.text.format.DateFormat;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.entities.WorklistTestData;

import java.util.ArrayList;

/**
 * @author Ruben Carmona
 * @project TFM - Analyzer Handler
 * @date 05/2016
 */
public class WorklistFullTestItemAdapter extends RecyclerView.Adapter<WorklistFullTestItemAdapter.WorklistFullTestViewHolder> {

    private TextView emptyView;
    private ArrayList<WorklistTestData> worklistTestDatas;

    final RecyclerView.AdapterDataObserver observer = new RecyclerView.AdapterDataObserver() {
        @Override
        public void onChanged() {
            super.onChanged();
            checkIfEmpty();
        }
    };

    public WorklistFullTestItemAdapter(ArrayList<WorklistTestData> worklistTestDataList) {

        this.worklistTestDatas = worklistTestDataList;
        this.registerAdapterDataObserver(observer);
    }

    @Override
    public WorklistFullTestViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View row = LayoutInflater.from(parent.getContext()).inflate(R.layout.list_full_worklist_tests_detail_item, parent, false);
        WorklistFullTestViewHolder pvh = new WorklistFullTestViewHolder(row);
        return pvh;
    }

    @Override
    public void onBindViewHolder(WorklistFullTestViewHolder holder, int position) {
        java.text.DateFormat dateFormat = DateFormat.getTimeFormat(holder.getmFinishedLayout().getContext());

        WorklistTestData aWorklistTestData = worklistTestDatas.get(position);


        holder.mTestCode.setText(aWorklistTestData.getCode());
        holder.mStartTime.setText(dateFormat.format(aWorklistTestData.getInitTime()));
        holder.mTestDescription.setText(aWorklistTestData.getDescription());
        holder.mRemainingTime.setText(aWorklistTestData.getMinutes() + " min");
        holder.mRemainingTests.setText(aWorklistTestData.getFeasibility() + "");

        if (aWorklistTestData.getFinishTime() != null) {

            holder.mBackgroundZero.setVisibility(View.VISIBLE);
            holder.mFinishedTestStatus.setVisibility(View.VISIBLE);
            holder.mFinishedLayout.setVisibility(View.VISIBLE);
            holder.mFinishedInitTime.setText(dateFormat.format(aWorklistTestData.getInitTime()));
            holder.mFinishedFinishTime.setText(dateFormat.format(aWorklistTestData.getFinishTime()));
            holder.mRemainingTests.setVisibility(View.GONE);
            holder.mRemainingTime.setVisibility(View.GONE);
            holder.mStartTime.setVisibility(View.GONE);
        } else {
            if (position > 0) {
                WorklistTestData previousWTD = worklistTestDatas.get(position - 1);
                if (previousWTD.getFinishTime() == null) {
                    holder.mBackgroundZero.setVisibility(View.VISIBLE);

                    holder.mFinishedTestStatus.setVisibility(View.GONE);
                    holder.mFinishedLayout.setVisibility(View.VISIBLE);
                    holder.mFinishedInitTime.setText(dateFormat.format(aWorklistTestData.getInitTime()));
                    holder.mFinishedInitTime.setVisibility(View.VISIBLE);
                    holder.mFinishedFinishTime.setVisibility(View.GONE);
                    holder.mRemainingTests.setVisibility(View.GONE);
                    holder.mRemainingTime.setVisibility(View.GONE);
                    holder.mStartTime.setVisibility(View.GONE);
                } else {
                    holder.mBackgroundZero.setVisibility(View.GONE);
                    holder.mFinishedTestStatus.setVisibility(View.GONE);
                    holder.mFinishedLayout.setVisibility(View.GONE);
                    holder.mRemainingTests.setVisibility(View.VISIBLE);
                    holder.mRemainingTime.setVisibility(View.VISIBLE);
                    holder.mStartTime.setVisibility(View.VISIBLE);
                }
            } else {
                holder.mBackgroundZero.setVisibility(View.GONE);
                holder.mFinishedTestStatus.setVisibility(View.GONE);
                holder.mFinishedLayout.setVisibility(View.GONE);
                holder.mRemainingTests.setVisibility(View.VISIBLE);
                holder.mRemainingTime.setVisibility(View.VISIBLE);
                holder.mStartTime.setVisibility(View.VISIBLE);
            }
        }
    }

    public void setEmptyView(TextView pEmptyView) {
        this.emptyView = pEmptyView;
    }

    private void checkIfEmpty() {
        if (emptyView != null) {
            emptyView.setVisibility(worklistTestDatas.size() > 0 ? View.GONE : View.VISIBLE);
        }
    }

    @Override
    public int getItemCount() {
        return worklistTestDatas.size();
    }

    static class WorklistFullTestViewHolder extends RecyclerView.ViewHolder {

        private LinearLayout mFinishedLayout;
        private TextView mFinishedInitTime;
        private TextView mFinishedFinishTime;
        private ImageView mFinishedTestStatus;

        private TextView mTestCode;
        private TextView mTestDescription;
        private TextView mRemainingTime;
        private TextView mStartTime;
        private TextView mRemainingTests;

        private TextView mBackgroundZero;

        public WorklistFullTestViewHolder(View itemView) {
            super(itemView);
            mFinishedLayout = (LinearLayout) itemView.findViewById(R.id.list_full_worklist_finished_layout);
            mFinishedInitTime = (TextView) itemView.findViewById(R.id.list_full_worklist_finished_init_time);
            mFinishedFinishTime = (TextView) itemView.findViewById(R.id.list_full_worklist_finished_finish_time);

            mTestCode = (TextView) itemView.findViewById(R.id.list_full_worklist_test_code);
            mTestDescription = (TextView) itemView.findViewById(R.id.list_full_worklist_test_description);
            mRemainingTime = (TextView) itemView.findViewById(R.id.list_full_worklist_tests_remaning_time);
            mStartTime = (TextView) itemView.findViewById(R.id.list_full_worklist_test_start_time);
            mRemainingTests = (TextView) itemView.findViewById(R.id.list_full_worklist_tests_remaining_feasibility);
            mFinishedTestStatus = (ImageView) itemView.findViewById(R.id.list_full_worklist_test_status_icon);

            mBackgroundZero = (TextView) itemView.findViewById(R.id.list_full_tests_background_zero);
        }

        public LinearLayout getmFinishedLayout() {
            return mFinishedLayout;
        }
    }
}