/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or 
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any 
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by 
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.adapter;

import android.support.v7.widget.RecyclerView;
import android.text.format.DateFormat;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.entities.WorklistTestData;

import java.util.ArrayList;
import java.util.Calendar;

/**
 * @author Ruben Carmona
 * @project TFM - Analyzer Handler
 * @date 05/2016
 */
public class WorklistTestItemAdapter extends RecyclerView.Adapter<WorklistTestItemAdapter.WorklistTestViewHolder> {

    private TextView emptyView;
    private ArrayList<WorklistTestData> worklistTestDatas;

    final RecyclerView.AdapterDataObserver observer = new RecyclerView.AdapterDataObserver() {
        @Override
        public void onChanged() {
            super.onChanged();
            checkIfEmpty();
        }
    };

    public WorklistTestItemAdapter(ArrayList<WorklistTestData> worklistTestDataList) {

        this.worklistTestDatas = worklistTestDataList;

        this.registerAdapterDataObserver(observer);
    }

    @Override
    public WorklistTestViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View row = LayoutInflater.from(parent.getContext()).inflate(R.layout.list_worklist_tests_detail_item, parent, false);
        WorklistTestViewHolder WorklistTestViewHolder = new WorklistTestViewHolder(row);
        return WorklistTestViewHolder;
    }

    @Override
    public void onBindViewHolder(WorklistTestViewHolder holder, int position) {
        WorklistTestData aWorklistTestData = worklistTestDatas.get(position);
        java.text.DateFormat dateFormat = DateFormat.getTimeFormat(holder.getmRemainingFeasibility().getContext());

        holder.mTestCode.setText(aWorklistTestData.getCode());
        holder.mTestDescription.setText(aWorklistTestData.getDescription());

        if (position == 0) {
            holder.mStartTime.setText(dateFormat.format(aWorklistTestData.getInitTime()));
            holder.mRemainingTime.setText(aWorklistTestData.getMinutes() + " min");
            holder.mRemainingFeasibility.setText(aWorklistTestData.getFeasibility() + "");
            holder.mRemainingTime.setVisibility(View.VISIBLE);
            holder.mFutureStart.setVisibility(View.GONE);
        } else {
            holder.mFutureStart.setText(dateFormat.format(getWhenStart(position).getTime()));
            holder.mRemainingTime.setVisibility(View.GONE);
            holder.mFutureStart.setVisibility(View.VISIBLE);
        }
        holder.mStartTime.setVisibility(position == 0 ? View.VISIBLE : View.GONE);
        holder.mRemainingFeasibility.setVisibility(position == 0 ? View.VISIBLE : View.GONE);
        holder.mBackground.setVisibility(position == 0 ? View.GONE : View.VISIBLE);
    }

    private Calendar getWhenStart(int pPosition) {

        WorklistTestData worklistTestData = worklistTestDatas.get(0);
        Calendar cal = Calendar.getInstance();
        cal.setTimeInMillis(worklistTestData.getInitTime().getTime());
        cal.add(Calendar.MINUTE, worklistTestData.getMinutes().intValue() + 5);

        for (int i = 1; i <= pPosition; i++) {
            worklistTestData = worklistTestDatas.get(i);
            cal.add(Calendar.MINUTE, worklistTestData.getMinutes().intValue() + 5);
        }
        return cal;
    }

    public void setEmptyView(TextView pEmptyView) {
        this.emptyView = pEmptyView;
    }

    private void checkIfEmpty() {
        if (emptyView != null) {
            emptyView.setVisibility(worklistTestDatas.size() > 0 ? View.GONE : View.VISIBLE);
        }
    }

    @Override
    public int getItemCount() {
        return worklistTestDatas.size();
    }

    static class WorklistTestViewHolder extends RecyclerView.ViewHolder {
        private TextView mTestCode;
        private TextView mTestDescription;
        private TextView mRemainingTime;
        private TextView mStartTime;
        private TextView mRemainingFeasibility;
        private TextView mBackground;
        private TextView mFutureStart;

        public WorklistTestViewHolder(View itemView) {
            super(itemView);
            mTestCode = (TextView) itemView.findViewById(R.id.detail_analyzer_description);
            mTestDescription = (TextView) itemView.findViewById(R.id.worklist_tests_description);
            mRemainingTime = (TextView) itemView.findViewById(R.id.worklist_tests_remaning_time);
            mStartTime = (TextView) itemView.findViewById(R.id.worklist_test_start_time);
            mFutureStart = (TextView) itemView.findViewById(R.id.worklist_test_future_start);
            mRemainingFeasibility = (TextView) itemView.findViewById(R.id.worklist_tests_remaining_feasibility);
            mBackground = (TextView) itemView.findViewById(R.id.worklist_tests_background_zero);
        }


        public TextView getmTestCode() {
            return mTestCode;
        }

        public TextView getmTestDescription() {
            return mTestDescription;
        }

        public TextView getmRemainingTime() {
            return mRemainingTime;
        }

        public TextView getmStartTime() {
            return mStartTime;
        }

        public TextView getmRemainingFeasibility() {
            return mRemainingFeasibility;
        }

    }
}