/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */
package com.uoc.tfm.analyzerhandler.database;

import android.support.v4.widget.SwipeRefreshLayout;

import com.uoc.tfm.analyzerhandler.adapter.AnalyzerItemAdapter;
import com.uoc.tfm.analyzerhandler.common.AppGlobals;
import com.uoc.tfm.analyzerhandler.entities.AnalyzerData;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;


/**
 * @author Rub
 * @project TFM - Analyzer Simulator
 * @filename AnalyzerDAO.java
 */
public class AnalyzersDAO extends MySQLDAO {

    public static final String GET_ALL = "GET_ALL";
    public static final String UPDATE_CURRENT = "UPDATE_CURRENT";

    private ArrayList<AnalyzerData> analyzersItems = null;
    private AnalyzerData analyzerData = null;
    private AnalyzerItemAdapter aArrayAdapter = null;
    private SwipeRefreshLayout aSwipeRefreshLayout;

    private String currentAction = "";

    public AnalyzersDAO(ArrayList<AnalyzerData> pAnalyzersItems, AnalyzerItemAdapter pArrayAdapter, SwipeRefreshLayout pSwipeRefreshLayout, String pAction) {
        this.analyzersItems = pAnalyzersItems;
        this.aArrayAdapter = pArrayAdapter;
        this.aSwipeRefreshLayout = pSwipeRefreshLayout;
        this.currentAction = pAction;
    }

    @Override
    protected Object doInBackground(Void... params) {
        if (this.analyzersItems != null) {
            return getAnalyzerList();
        } else {
            return new AnalyzerData();
        }
    }

    public ArrayList<AnalyzerData> getAnalyzerList() {
        ArrayList<AnalyzerData> aAnalyzerList = new ArrayList<>();

        try {
            initConnection();
            stmt = AppGlobals.conn.createStatement();
            switch (this.currentAction) {
                case GET_ALL:
                    getAll(aAnalyzerList);
                    break;
                case UPDATE_CURRENT:
                    break;
                default:
                    break;
            }
        } catch (SQLException e) {
            System.err.println(e.getMessage());
        } finally {
            super.close();
        }

        return aAnalyzerList;
    }

    private void getAll(ArrayList<AnalyzerData> aAnalyzerList) throws SQLException {
        resultSet = stmt.executeQuery("SELECT * FROM analyzer_simulator.Analyzer");

        while (resultSet.next()) {
            AnalyzerData aAnalyzer = new AnalyzerData();
            aAnalyzer.fromResultSet(resultSet);

            String strSQL = "SELECT n.id, n.description, n.kind, an.date, an.Analyzer_id ";
            strSQL += "FROM Notifications n, AnalyzerNotifications an ";
            strSQL += "WHERE n.id = an.Notifications_id AND an.Analyzer_id = ? ORDER BY an.date DESC";

            preparedStmt = AppGlobals.conn.prepareStatement(strSQL);
            preparedStmt.setInt(1, aAnalyzer.getId().intValue());

            ResultSet tmpResultSet = preparedStmt.executeQuery();
            aAnalyzer.notificationsFromResultSet(tmpResultSet);

            strSQL = "SELECT at.*, t.* FROM AnalyzerTests at, Tests t WHERE at.Analyzer_id = ? AND at.Tests_id = t.id";

            preparedStmt = AppGlobals.conn.prepareStatement(strSQL);
            preparedStmt.setInt(1, aAnalyzer.getId().intValue());

            tmpResultSet = preparedStmt.executeQuery();

            aAnalyzer.configuredTestFromResultSet(tmpResultSet);

            strSQL = "SELECT w.id, w.Analyzer_id, w.description, w.inittime AS winittime, w.finishtime AS wfinishtime ";
            strSQL += "FROM Worklist w WHERE w.Analyzer_id = ? ORDER BY w.inittime DESC";

            preparedStmt = AppGlobals.conn.prepareStatement(strSQL);
            preparedStmt.setInt(1, aAnalyzer.getId().intValue());

            tmpResultSet = preparedStmt.executeQuery();

            aAnalyzer.worklistsFromResultSet(tmpResultSet);

            strSQL = "SELECT wa.*, t.*, a.feasibility FROM Worklist w, WorklistAnalyzerTests wa, Tests t, AnalyzerTests a ";
            strSQL += "WHERE w.Analyzer_id = ? AND w.Analyzer_id = wa.Worklist_Analyzer_id  AND w.id = wa.Worklist_id ";
            strSQL += "AND wa.Tests_id = t.id AND a.Analyzer_id = w.Analyzer_id AND a.Tests_id = t.id ORDER BY wa.finishtime DESC, wa.inittime ASC";

            preparedStmt = AppGlobals.conn.prepareStatement(strSQL);
            preparedStmt.setInt(1, aAnalyzer.getId().intValue());

            tmpResultSet = preparedStmt.executeQuery();

            aAnalyzer.worklistsTestFromResultSet(tmpResultSet);

            aAnalyzerList.add(aAnalyzer);
        }
    }

    @Override
    protected void onPostExecute(Object pObject) {
        if (GET_ALL.equals(this.currentAction)) {
            this.analyzersItems.clear();
            this.analyzersItems.addAll((ArrayList<AnalyzerData>) pObject);
            this.aArrayAdapter.notifyDataSetChanged();
            this.aSwipeRefreshLayout.setRefreshing(false);
        }
    }

    @Override
    protected void onCancelled() {
        aSwipeRefreshLayout.setRefreshing(false);
    }
}
