/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */
package com.uoc.tfm.analyzerhandler.database;

import android.os.AsyncTask;

import com.uoc.tfm.analyzerhandler.common.AppGlobals;

import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;


/**
 * @author Rub
 * @project TFM - Analyzer Simulator
 * @filename MySQLAccess.java
 */
public abstract class MySQLDAO extends AsyncTask<Void, Void, Object> {

    protected Statement stmt = null;
    protected PreparedStatement preparedStmt = null;
    protected ResultSet resultSet = null;

    protected static void initConnection() throws SQLException {
        if (AppGlobals.conn == null || (AppGlobals.conn != null && AppGlobals.conn.isClosed())) {
            try {
                // This will load the MySQL driver, each DB has its own driver
                Class.forName("com.mysql.jdbc.Driver");
                // Setup the connection with the DB
                DriverManager.setLoginTimeout(10);
                AppGlobals.conn = DriverManager.getConnection("jdbc:mysql://" + AppGlobals.SERVER_IP + ":3306" + AppGlobals.DATABASE_NAME, "tfm_user", "tfm_user");
            } catch (ClassNotFoundException | SQLException e) {
                System.err.println(e.getMessage());
//        JOptionPane.showMessageDialog(AnalyzerSimulatorWindow.mainFrame, e.getLocalizedMessage(), "Connection Error", JOptionPane.ERROR_MESSAGE);
                System.exit(-1);
            }
        }
    }

    protected void closeStatement() {
        if (stmt != null) {
            try {
                stmt.close();
            } catch (SQLException e) {
                System.err.println(e.getMessage());
            }
        }
    }

    public void readDataBase() throws Exception {
        try {

            // Statements allow to issue SQL queries to the database
            stmt = AppGlobals.conn.createStatement();
            // Result set get the result of the SQL query
            resultSet = stmt.executeQuery("select * from analyzer_simulator.comments");
            writeResultSet(resultSet);

            // PreparedStatements can use variables and are more efficient
            preparedStmt = AppGlobals.conn.prepareStatement("insert into  analyzer_simulator.comments values (default, ?, ?, ?, ? , ?, ?)");
            // "myuser, webpage, datum, summery, COMMENTS from analyzer_simulator.comments");
            // Parameters start with 1
            preparedStmt.setString(1, "Test");
            preparedStmt.setString(2, "TestEmail");
            preparedStmt.setString(3, "TestWebpage");
            preparedStmt.setDate(4, new java.sql.Date(2009, 12, 11));
            preparedStmt.setString(5, "TestSummary");
            preparedStmt.setString(6, "TestComment");
            preparedStmt.executeUpdate();

            preparedStmt = AppGlobals.conn.prepareStatement("SELECT myuser, webpage, datum, summery, COMMENTS from analyzer_simulator.comments");
            resultSet = preparedStmt.executeQuery();
            writeResultSet(resultSet);

            // Remove again the insert comment
            preparedStmt = AppGlobals.conn.prepareStatement("delete from analyzer_simulator.comments where myuser= ? ; ");
            preparedStmt.setString(1, "Test");
            preparedStmt.executeUpdate();

            resultSet = stmt.executeQuery("select * from analyzer_simulator.comments");
            writeMetaData(resultSet);

        } catch (Exception e) {
            throw e;
        } finally {
            close();
        }

    }

    private void writeMetaData(ResultSet resultSet) throws SQLException {
        // Now get some metadata from the database
        // Result set get the result of the SQL query

        System.out.println("The columns in the table are: ");

        System.out.println("Table: " + resultSet.getMetaData().getTableName(1));
        for (int i = 1; i <= resultSet.getMetaData().getColumnCount(); i++) {
            System.out.println("Column " + i + " " + resultSet.getMetaData().getColumnName(i));
        }
    }

    private void writeResultSet(ResultSet resultSet) throws SQLException {
        // ResultSet is initially before the first data set
        while (resultSet.next()) {
            // It is possible to get the columns via name
            // also possible to get the columns via the column number
            // which starts at 1
            // e.g. resultSet.getSTring(2);
            String user = resultSet.getString("myuser");
            String website = resultSet.getString("webpage");
            String summery = resultSet.getString("summery");
            Date date = resultSet.getDate("datum");
            String comment = resultSet.getString("comments");
            System.out.println("User: " + user);
            System.out.println("Website: " + website);
            System.out.println("Summery: " + summery);
            System.out.println("Date: " + date);
            System.out.println("Comment: " + comment);
        }
    }

    // You need to close the resultSet
    protected void close() {
        try {
            if (resultSet != null) {
                resultSet.close();
            }

            if (stmt != null) {
                stmt.close();
            }

            if (preparedStmt != null) {
                preparedStmt.close();
            }
            if (AppGlobals.conn != null) {
                AppGlobals.conn.close();
                AppGlobals.conn = null;
            }
        } catch (Exception e) {

        }
    }

}
