/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */
package com.uoc.tfm.analyzerhandler.database;

import android.support.v4.widget.SwipeRefreshLayout;

import com.uoc.tfm.analyzerhandler.adapter.NotificationsItemAdapter;
import com.uoc.tfm.analyzerhandler.common.AppGlobals;
import com.uoc.tfm.analyzerhandler.entities.NotificationData;

import java.sql.SQLException;
import java.util.ArrayList;


/**
 * @author Rub
 * @project TFM - Analyzer Simulator
 * @filename TestDAO.java
 */
public class NotificationsDAO extends MySQLDAO {

    ArrayList<NotificationData> notificationDataArrayList;
    private SwipeRefreshLayout aSwipeRefreshLayout;
    private Integer analyzerID;
    private String kind;
    private NotificationsItemAdapter notificationsItemAdapter;

    public NotificationsDAO(Integer pAnalyzerID, String pKind, ArrayList<NotificationData> pNotificationDataList, NotificationsItemAdapter pArrayAdapter, SwipeRefreshLayout pSwipeRefreshLayout) {
        this.analyzerID = pAnalyzerID;
        this.kind = pKind;
        this.notificationDataArrayList = pNotificationDataList;
        this.notificationsItemAdapter = pArrayAdapter;
        this.aSwipeRefreshLayout = pSwipeRefreshLayout;
    }

    @Override
    protected Object doInBackground(Void... params) {
        if (this.notificationDataArrayList != null) {
            return updateNotifications();
        } else {
            return null;
        }
    }

    private ArrayList<NotificationData> updateNotifications() {

        ArrayList<NotificationData> aNotificationList = new ArrayList<>();

        try {
            initConnection();
            preparedStmt = AppGlobals.conn.prepareStatement("SELECT DISTINCT n.*, an.* FROM Notifications n, AnalyzerNotifications an WHERE an.Analyzer_id = ? AND n.kind = ? AND an.Notifications_id = n.id");
            preparedStmt.setInt(1, analyzerID);
            preparedStmt.setString(2, kind);
            resultSet = preparedStmt.executeQuery();

            while (resultSet.next()) {
                NotificationData aNotification = new NotificationData();
                aNotification.fromResultSet(resultSet, true);
                aNotificationList.add(aNotification);
            }
        } catch (SQLException e) {
            System.err.println(e.getMessage());
        } finally {
            this.close();
        }

        return aNotificationList;

    }

    @Override
    protected void onPostExecute(Object pObject) {
        if (pObject != null) {
            this.notificationDataArrayList.clear();
            this.notificationDataArrayList.addAll((ArrayList<NotificationData>) pObject);
            this.notificationsItemAdapter.notifyDataSetChanged();
        }
        this.aSwipeRefreshLayout.setRefreshing(false);
    }

    @Override
    protected void onCancelled() {
        aSwipeRefreshLayout.setRefreshing(false);
    }
}
