/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */
package com.uoc.tfm.analyzerhandler.database;

import android.support.v4.widget.SwipeRefreshLayout;

import com.uoc.tfm.analyzerhandler.adapter.AnalyzerTestConfigurationItemAdapter;
import com.uoc.tfm.analyzerhandler.common.AppGlobals;
import com.uoc.tfm.analyzerhandler.entities.AnalyzerData;
import com.uoc.tfm.analyzerhandler.entities.TestData;

import java.sql.SQLException;
import java.util.ArrayList;

/**
 * @author Rub
 * @project TFM - Analyzer Simulator
 * @filename TestDAO.java
 */
public class TestsDAO extends MySQLDAO {
    private ArrayList<TestData> testDataArrayList;
    private AnalyzerTestConfigurationItemAdapter analyzerTestConfigurationItemAdapter = null;
    private SwipeRefreshLayout aSwipeRefreshLayout;
    private Integer analyzerID;

    public TestsDAO(Integer pAnalyzerID, ArrayList<TestData> pTestDataList, AnalyzerTestConfigurationItemAdapter pArrayAdapter, SwipeRefreshLayout pSwipeRefreshLayout) {
        this.analyzerID = pAnalyzerID;
        this.testDataArrayList = pTestDataList;
        this.analyzerTestConfigurationItemAdapter = pArrayAdapter;
        this.aSwipeRefreshLayout = pSwipeRefreshLayout;
    }

    @Override
    protected Object doInBackground(Void... params) {
        if (this.testDataArrayList != null) {
            return updateTests();
        } else {
            return null;
        }
    }

    private ArrayList<TestData> updateTests() {
        AnalyzerData analyzerData = new AnalyzerData();
        try {
            initConnection();
            stmt = AppGlobals.conn.createStatement();
            String strSQL = "SELECT at.*, t.* FROM AnalyzerTests at, Tests t WHERE at.Analyzer_id = ? AND at.Tests_id = t.id";

            preparedStmt = AppGlobals.conn.prepareStatement(strSQL);
            preparedStmt.setInt(1, analyzerID);

            resultSet = preparedStmt.executeQuery();

            analyzerData.configuredTestFromResultSet(resultSet);

        } catch (SQLException e) {
            System.err.println(e.getMessage());
        } finally {
            super.close();
        }

        return analyzerData.getConfiguredTests();
    }

    @Override
    protected void onPostExecute(Object pObject) {
        if (pObject != null) {
            this.testDataArrayList.clear();
            this.testDataArrayList.addAll((ArrayList<TestData>) pObject);
            this.analyzerTestConfigurationItemAdapter.notifyDataSetChanged();
        }
        this.aSwipeRefreshLayout.setRefreshing(false);
    }

    @Override
    protected void onCancelled() {
        aSwipeRefreshLayout.setRefreshing(false);
    }
}
