/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */
package com.uoc.tfm.analyzerhandler.database;

import android.support.v4.widget.SwipeRefreshLayout;

import com.uoc.tfm.analyzerhandler.adapter.WorklistFullTestItemAdapter;
import com.uoc.tfm.analyzerhandler.adapter.WorklistTestItemAdapter;
import com.uoc.tfm.analyzerhandler.common.AppGlobals;
import com.uoc.tfm.analyzerhandler.entities.AnalyzerData;
import com.uoc.tfm.analyzerhandler.entities.WorklistData;
import com.uoc.tfm.analyzerhandler.entities.WorklistTestData;

import java.sql.SQLException;
import java.util.ArrayList;


/**
 * @author Rub
 * @project TFM - Analyzer Simulator
 * @filename TestDAO.java
 */
public class WorklistDAO extends MySQLDAO {

    private ArrayList<WorklistTestData> worklistTestDataList;
    private WorklistTestItemAdapter aWorklistTestItemAdapter = null;
    private WorklistFullTestItemAdapter aFullWorklistItemAdapter = null;
    private SwipeRefreshLayout aSwipeRefreshLayout;
    private Integer analyzerID;

    public WorklistDAO(Integer pAnalyzerID, ArrayList<WorklistTestData> pWorklistTestData, WorklistTestItemAdapter pArrayAdapter, SwipeRefreshLayout pSwipeRefreshLayout) {
        this.analyzerID = pAnalyzerID;
        this.worklistTestDataList = pWorklistTestData;
        this.aWorklistTestItemAdapter = pArrayAdapter;
        this.aFullWorklistItemAdapter = null;
        this.aSwipeRefreshLayout = pSwipeRefreshLayout;
    }

    public WorklistDAO(Integer pAnalyzerID, ArrayList<WorklistTestData> pWorklistTestData, WorklistFullTestItemAdapter pArrayAdapter, SwipeRefreshLayout pSwipeRefreshLayout) {
        this.analyzerID = pAnalyzerID;
        this.worklistTestDataList = pWorklistTestData;
        this.aFullWorklistItemAdapter = pArrayAdapter;
        this.aWorklistTestItemAdapter = null;
        this.aSwipeRefreshLayout = pSwipeRefreshLayout;
    }

    @Override
    protected Object doInBackground(Void... params) {
        if (this.worklistTestDataList != null) {
            return updateWorklistTests();
        } else {
            return null;
        }
    }

    public WorklistData updateWorklistTests() {

        AnalyzerData analyzerData = new AnalyzerData();
        try {
            initConnection();
            stmt = AppGlobals.conn.createStatement();
            String strSQL = "SELECT wa.*, t.*, a.feasibility FROM Worklist w, WorklistAnalyzerTests wa, Tests t, AnalyzerTests a ";
            strSQL += "WHERE w.Analyzer_id = ? AND w.Analyzer_id = wa.Worklist_Analyzer_id  AND w.id = wa.Worklist_id ";
            strSQL += "AND wa.Tests_id = t.id AND a.Analyzer_id = w.Analyzer_id AND a.Tests_id = t.id ORDER BY wa.finishtime DESC, wa.inittime ASC";

            preparedStmt = AppGlobals.conn.prepareStatement(strSQL);
            preparedStmt.setInt(1, analyzerID.intValue());

            resultSet = preparedStmt.executeQuery();
            analyzerData.setWorklist(new ArrayList<WorklistData>());
            analyzerData.getWorklist().add(new WorklistData());
            analyzerData.worklistsTestFromResultSet(resultSet);

        } catch (SQLException e) {
            System.err.println(e.getMessage());
        } finally {
            super.close();
        }

        return analyzerData.getWorklist().get(0);
    }

    @Override
    protected void onPostExecute(Object pObject) {
        if (pObject != null) {
            if (aWorklistTestItemAdapter != null) {
                this.worklistTestDataList.clear();
                this.worklistTestDataList.addAll(((WorklistData) pObject).getCurrentWorklistTest());
                this.aWorklistTestItemAdapter.notifyDataSetChanged();
            } else if (aFullWorklistItemAdapter != null) {
                this.worklistTestDataList.clear();
                this.worklistTestDataList.addAll(((WorklistData) pObject).getWorklistTestList());
                this.aFullWorklistItemAdapter.notifyDataSetChanged();
            }
            this.aSwipeRefreshLayout.setRefreshing(false);
        }
    }

    @Override
    protected void onCancelled() {
        aSwipeRefreshLayout.setRefreshing(false);
    }
}
