/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */
package com.uoc.tfm.analyzerhandler.entities;

import java.io.Serializable;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Random;

/**
 * @project TFM - Analyzer Simulator
 * @author Ruben Carmona
 * @filename Notification.java
 */

public class NotificationData implements Serializable
{
  private static final String TAG = NotificationData.class.getSimpleName();

  public static final String KIND_ALARM = "A";
  public static final String KIND_WARNING = "W";
  public static final String KIND_TESTFEASIBILITY_INFO = "I";

  private Integer id;
  private String description;
  private String kind;
  private Integer analyzerID;
  private Timestamp time;

  public NotificationData()
  {

  }

  public NotificationData(NotificationData pNotif)
  {
    this.id = pNotif.getId();
    this.kind = pNotif.getKind();
    this.description = pNotif.getDescription();
    this.time = new Timestamp(System.currentTimeMillis());
  }

  public NotificationData(TestData aTest)
  {
    this.id = new Random().nextInt(1000);
    this.kind = KIND_TESTFEASIBILITY_INFO;
    this.description = aTest.getCode() + " - " + aTest.getDescription();
    this.time = new Timestamp(System.currentTimeMillis());
  }

  public Integer getId()
  {
    return id;
  }

  public void setId(Integer id)
  {
    this.id = id;
  }

  public String getDescription()
  {
    return description;
  }

  public void setDescription(String description)
  {
    this.description = description;
  }

  public String getKind()
  {
    return kind;
  }

  public void setKind(String kind)
  {
    this.kind = kind;
  }

  public Integer getAnalyzerID()
  {
    return analyzerID;
  }

  public void setAnalyzerID(Integer analyzerID)
  {
    this.analyzerID = analyzerID;
  }

  public Timestamp getTime()
  {
    return time;
  }

  public void setTime(Timestamp time)
  {
    this.time = time;
  }

  @Override
  public String toString()
  {
    return this.getDescription();
  }

  public void fromResultSet(ResultSet pRS, boolean forAnalyzer)
  {
    try
    {
      this.setId(pRS.getInt("id"));
      this.setDescription(pRS.getString("description"));
      this.setKind(pRS.getString("kind"));

      if (forAnalyzer)
      {
        this.setAnalyzerID(pRS.getInt("Analyzer_id"));
        this.setTime(pRS.getTimestamp("date"));
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
  }

  public static ArrayList<NotificationData> createFakeAlarmsNotification()
  {
    ArrayList<NotificationData> aArrayList = new ArrayList<>();
    for (int i = 0; i < 10; i++)
    {
      NotificationData notif = new NotificationData();
      notif.setId(new Random().nextInt(1000));
      notif.setKind(KIND_ALARM);
      notif.setDescription("Alarm description " + i);
      notif.setTime(new Timestamp(System.currentTimeMillis()));

      aArrayList.add(notif);
    }
    return aArrayList;
  }

  public static ArrayList<NotificationData> createFakeWarningNotification()
  {
    ArrayList<NotificationData> aArrayList = new ArrayList<>();

    for (int i = 0; i < 10; i++)
    {
      NotificationData notif = new NotificationData();
      notif.setId(new Random().nextInt(1000));
      notif.setKind(KIND_WARNING);
      notif.setDescription("Warning description " + i);
      notif.setTime(new Timestamp(System.currentTimeMillis()));

      aArrayList.add(notif);
    }

    return aArrayList;
  }
}
