/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or 
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any 
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by 
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.fragments;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.activities.NotificationsAnalyzerActivity;
import com.uoc.tfm.analyzerhandler.adapter.NotificationsItemAdapter;
import com.uoc.tfm.analyzerhandler.database.NotificationsDAO;
import com.uoc.tfm.analyzerhandler.entities.AnalyzerData;
import com.uoc.tfm.analyzerhandler.entities.NotificationData;

import java.util.ArrayList;

/**
 * @author Ruben Carmona
 * @project TFM - Analyzer Handler
 * @date 05/2016
 */
public class NotificationsInfoFragment extends Fragment {
    private SwipeRefreshLayout swipeRefreshLayout;
    private NotificationsItemAdapter mNotificationsItemAdapter;
    private NotificationsDAO notificationsDAO;
    private AnalyzerData analyzerData;
    private ArrayList<NotificationData> mNotificationDataArrayList;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.notifications_fragment_info, container, false);
        analyzerData = ((NotificationsAnalyzerActivity) getActivity()).getAnalyzerData();

        swipeRefreshLayout = (SwipeRefreshLayout) view.findViewById(R.id.notifications_fragment_info_swipeLayout);
        swipeRefreshLayout.setColorSchemeResources(
                R.color.colorAccent,
                R.color.lightBackground,
                R.color.hintColorLight);
        swipeRefreshLayout.setOnRefreshListener(
                new SwipeRefreshLayout.OnRefreshListener() {
                    @Override
                    public void onRefresh() {
                        notificationsDAO = new NotificationsDAO(analyzerData.getId(), NotificationData.KIND_TESTFEASIBILITY_INFO, mNotificationDataArrayList, mNotificationsItemAdapter, swipeRefreshLayout);
                        notificationsDAO.execute();
                    }
                });

        TextView emptyMessage = (TextView) view.findViewById(R.id.notifications_fragment_info_empty_view);
        mNotificationDataArrayList = analyzerData.getNotificationListByKind(NotificationData.KIND_TESTFEASIBILITY_INFO);
        RecyclerView recyclerView = (RecyclerView) view.findViewById(R.id.notifications_fragment_info_recyclerView);
        mNotificationsItemAdapter = new NotificationsItemAdapter(mNotificationDataArrayList);
        mNotificationsItemAdapter.setEmptyView(emptyMessage);

        assert recyclerView != null;
        recyclerView.setAdapter(mNotificationsItemAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(view.getContext()));
        return view;
    }
}
