/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.util;

import android.content.Context;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.util.Log;


public class SharePreferenceUtil {

    private static final String TAG = SharePreferenceUtil.class.getSimpleName();

    public static boolean setStringValue(Context context, String key, String value) {
        boolean result = false;
        try {
            SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
            SharedPreferences.Editor editor = settings.edit();
            if(value!=null)
                editor.putString(key, value);
            else
                editor.remove(key);
            editor.commit();
            result = true;
        }catch(Exception e){
            Log.e(TAG, "Exception: " + e.getMessage());
        }
        return result;
    }

    public static boolean setLongValue(Context context, String key, Long value) {
        boolean result = false;
        try {
            SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
            SharedPreferences.Editor editor = settings.edit();
            if(value!=null)
                editor.putLong(key, value);
            else
                editor.remove(key);
            editor.commit();
            result = true;
        }catch(Exception e){
            Log.e(TAG, "Exception: " + e.getMessage());
        }
        return result;
    }

    /**
     * Method to remove Long value from SharedPreferences
     *
     * @param context
     * @param key
     * @return true if value has been removed correctly
     */
    public static boolean removeLongValue(Context context, String key){
        return setLongValue(context, key, null);
    }

    /**
     * Method to get a Long value from SharedPreferences (without default value).
     *
     * @param context
     * @param key
     * @return Long value
     */
    public static long getLongValue(Context context, String key) {
        SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
        return settings.getLong(key, -1);
    }

    public static String getStringValue(Context context, String key) {
        SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
        return settings.getString(key, "");
    }

    /**
     * Method to get Long value from SharedPreferences.
     *
     * @param context
     * @param key
     * @param defaultValue
     * @return Long value
     */
    public static long getLongValue(Context context, String key, Long defaultValue) {
        SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
        return settings.getLong(key, defaultValue);
    }
}
