/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita.
 * All rights reserved. The total or partial reproduction of this work it is 
 * forbidden by any means or process, including printing, reprography, 
 * microfilm, computer processing or any other system, as well as the 
 * distribution of copies by rental and lending, without the written 
 * permission of the author or the limits authorized by the law of 
 * intellectual property.
 *******************************************************************************/
package com.tfm_uoc.components;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.BorderFactory;
import javax.swing.JLabel;

import com.tfm_uoc.common.AImageUtil;
import com.tfm_uoc.common.UIUtils;
import com.tfm_uoc.entities.AnalyzerData;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename AAnalyzerDetailLabel.java
 */
public class AAnalyzerDetailLabel extends JLabel
{

  private AnalyzerData analyzer = null;
  private boolean forList = false;

  public AAnalyzerDetailLabel()
  {

  }

  public AAnalyzerDetailLabel(AnalyzerData pAnalyzer, boolean pForList)
  {
    this.analyzer = pAnalyzer;
    this.forList = pForList;
    init();
  }

  private void init()
  {
    this.setOpaque(true);
    this.setForeground(UIUtils.foreColorDark);
    this.setBorder(BorderFactory.createEmptyBorder(5, 1, 5, 0));
  }

  public void setAnalyzer(AnalyzerData analyzer)
  {
    this.analyzer = analyzer;
    this.updateUI();
  }

  public AnalyzerData getAnalyzer()
  {
    return analyzer;
  }

  public boolean isForList()
  {
    return forList;
  }

  @Override
  public Dimension getPreferredSize()
  {
    if (forList)
    {
      return new Dimension(200, 100);
    }
    else
    {
      return new Dimension(400, 100);
    }
  }

  @Override
  public void paint(Graphics g)
  {
    Graphics2D g2d = (Graphics2D) g;
    RenderingHints oldRenderingHints = g2d.getRenderingHints();

    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

    g2d.setColor(this.getBackground());
    g2d.fillRect(0, 2, this.getWidth(), this.getHeight() - 2);

    if (this.forList)
    {
      g2d.setColor(UIUtils.foreColorDark);
      g2d.drawString(analyzer.getCode(), 25, 25);

      g2d.drawString(analyzer.getDescription(), 25, 50);

      g2d.drawImage(AImageUtil.alarmIcon.getImage(), 25, 65, null);
      g2d.drawImage(AImageUtil.warningIcon.getImage(), 75, 65, null);
      g2d.drawImage(AImageUtil.infoIcon.getImage(), 125, 65, null);

      g2d.setFont(this.getFont().deriveFont(12f));

      g2d.setColor(UIUtils.errorColor);
      g2d.drawString(analyzer.getAlarms() + "", 50, 80);

      g2d.setColor(UIUtils.warningColor);
      g2d.drawString(analyzer.getWarnings() + "", 100, 80);

      g2d.setColor(UIUtils.infoColor);
      g2d.drawString(analyzer.getInfo() + "", 150, 80);

      if (this.analyzer.getConnected())
      {
        g2d.drawImage(AImageUtil.connectedIcon.getImage(), this.getWidth() - AImageUtil.connectedIcon.getIconWidth() - 5, 5, null);
      }
      else
      {
        g2d.drawImage(AImageUtil.disconnectedIcon.getImage(), this.getWidth() - AImageUtil.disconnectedIcon.getIconWidth() - 5, 5, null);
      }

      g2d.setColor(this.analyzer.getAlarms() > 0 ? UIUtils.errorColor : this.analyzer.getWarnings() > 0 ? UIUtils.warningColor : this.analyzer.getConnected() ? UIUtils.okColor : Color.LIGHT_GRAY);
      g2d.fillRect(0, 2, 20, this.getHeight() - 2);

      g2d.setPaint(UIUtils.overlayColor);

      if (this.getBackground().equals(UIUtils.nonSelectedBackground))
      {
        g2d.fillRect(0, 2, this.getWidth(), this.getHeight() - 2);
      }
    }
    else
    {
      Font originalFont = g.getFont();
      int X_POS = 25;
      int Y_FLAGS_POS = 80;
      int Y_IMAGE_POS = 65;

      FontMetrics fm = g.getFontMetrics();

      g2d.setColor(UIUtils.selectedBackground);
      g2d.fillRect(0, 0, this.getWidth(), this.getHeight());

      g2d.setFont(this.getFont().deriveFont(20f));

      g2d.setColor(UIUtils.foreColorDark);
      g2d.drawString(analyzer.getCode() + " - " + analyzer.getDescription(), X_POS, 25);

      g2d.setColor(UIUtils.foreColor);
      g2d.drawString("IP: " + analyzer.getIp(), X_POS, 50);

      g2d.setFont(originalFont);

      g2d.drawImage(AImageUtil.alarmIcon.getImage(), X_POS, Y_IMAGE_POS, null);
      g2d.setColor(UIUtils.errorColor);

      X_POS += AImageUtil.alarmIcon.getIconWidth() + 10;
      g2d.drawString(analyzer.getAlarms() + " errors", X_POS, Y_FLAGS_POS);

      X_POS += fm.stringWidth(analyzer.getAlarms() + " errors") + 20;

      g2d.drawImage(AImageUtil.warningIcon.getImage(), X_POS, Y_IMAGE_POS, null);
      g2d.setColor(UIUtils.warningColor);
      X_POS += AImageUtil.alarmIcon.getIconWidth() + 10;

      g2d.drawString(analyzer.getWarnings() + " warnings", X_POS, Y_FLAGS_POS);

      X_POS += fm.stringWidth(analyzer.getWarnings() + " warnings") + 20;
      g2d.drawImage(AImageUtil.infoIcon.getImage(), X_POS, Y_IMAGE_POS, null);
      g2d.setColor(UIUtils.infoColor);

      X_POS += AImageUtil.infoIcon.getIconWidth() + 10;
      g2d.drawString(analyzer.getInfo() + " feasibility warnings", X_POS, Y_FLAGS_POS);

      g2d.setColor(UIUtils.foreColor);
      g2d.drawLine(0, this.getHeight() - 1, this.getWidth(), this.getHeight() - 1);
    }

    g2d.setRenderingHints(oldRenderingHints);
  }

}
