/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.components;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import com.tfm_uoc.common.AImageUtil;
import com.tfm_uoc.common.Common;
import com.tfm_uoc.common.PushNotifications;
import com.tfm_uoc.common.UIUtils;
import com.tfm_uoc.common.WorklistRunnable;
import com.tfm_uoc.database.AnalyzersDAO;
import com.tfm_uoc.database.NotificationsDAO;
import com.tfm_uoc.database.WorklistDAO;
import com.tfm_uoc.entities.AnalyzerData;
import com.tfm_uoc.entities.NotificationData;
import com.tfm_uoc.entities.WorklistTestData;
import com.tfm_uoc.main.AddTestWorklistDialog;
import com.tfm_uoc.main.AnalyzerSimulatorWindow;
import com.tfm_uoc.main.CreateTestDialog;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename AAnalyzerPanel.java
 */
public class AAnalyzerPanel extends JPanel
{
  private AnalyzerData analyzer;
  private ACheckbox checkbox;
  private JPanel topPanel = new JPanel(new BorderLayout())
  {

    @Override
    public void paint(Graphics g)
    {
      super.paint(g);

      g.setColor(UIUtils.foreColor);
      g.drawLine(0, this.getHeight() - 1, this.getWidth(), this.getHeight() - 1);
    }

  };
  private JPanel cardPanel, tabsPanel, tabsLabelPanel, centerButtonsPanel, fullButtonsPanel, westButtonsPanel;
  private MyTabLabel tabWorklist, tabNotifications, tabTestConfiguration;
  private CardLayout cardLayout = new CardLayout();
  private String strConnected = "  Connected  ", strDisconnected = "Disconnected";
  private String strWorklist = "Worklist", strNotifications = "Notifications", strTestConfiguration = "Test Configuration";
  private AButton btnAddTest, btnRemoveTest, btnCreateTest, btnStartResetWorklist;
  private AAnalyzerDetailLabel aAnalyzerDetailLabel;
  private WorklistTable aWorklistTable;
  private NotificationsPanel aNotificationsPanel;
  private TestConfigurationTable aTestConfigurationTable;

  public AAnalyzerPanel(AnalyzerData pAnalyzer)
  {
    super(new BorderLayout());

    this.analyzer = pAnalyzer;
    init();
  }

  private void init()
  {
    topPanel.removeAll();

    this.setBackground(Color.WHITE);
    this.setOpaque(true);
    checkbox = new ACheckbox();
    checkbox.setText(this.analyzer.getConnected() ? strConnected : strDisconnected);
    checkbox.setSelected(this.analyzer.getConnected());
    checkbox.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    checkbox.setOpaque(false);
    checkbox.setFocusable(false);

    UIUtils.setFixedSize(checkbox, 100, 50);

    ActionListener actionListener = new ActionListener()
    {
      public void actionPerformed(ActionEvent actionEvent)
      {
        AbstractButton abstractButton = (AbstractButton) actionEvent.getSource();
        boolean selected = abstractButton.getModel().isSelected();
        AnalyzersDAO aDAO = new AnalyzersDAO();
        NotificationsDAO notifDAO = new NotificationsDAO();
        if (aDAO.updateConnectionStatus(analyzer.getId(), selected))
        {
          analyzer.setConnected(selected);
          checkbox.setText(selected ? strConnected : strDisconnected);
          ((AnalyzerSimulatorWindow) AnalyzerSimulatorWindow.mainFrame).updateListUI();
          NotificationData notifData = new NotificationData();
          notifData.setKind(NotificationData.KIND_ANALYZER_CONNECTION);
          notifData.setAnalyzerID(analyzer.getId());
          notifData.setAnalyzerCode(analyzer.getCode());
          notifData.setDescription(checkbox.getText());
          notifDAO.createAnalyzerNotification(notifData);

          PushNotifications.sendNotification(notifData);
        }

      }
    };
    checkbox.addActionListener(actionListener);

    topPanel.setBackground(Color.WHITE);
    topPanel.setOpaque(true);

    aAnalyzerDetailLabel = new AAnalyzerDetailLabel(this.analyzer, false);

    topPanel.add(aAnalyzerDetailLabel, BorderLayout.CENTER);
    topPanel.add(checkbox, BorderLayout.EAST);

    createTabs();

    this.add(topPanel, BorderLayout.NORTH);
    this.add(tabsPanel, BorderLayout.CENTER);
  }

  public void updateAnalyzer(AnalyzerData pAnalyzer)
  {
    this.analyzer = pAnalyzer;
    checkbox.setText(pAnalyzer.getConnected() ? strConnected : strDisconnected);
    checkbox.setSelected(pAnalyzer.getConnected());
    aAnalyzerDetailLabel.setAnalyzer(this.analyzer);
    aWorklistTable.updateAnalyzer(this.analyzer);
    aNotificationsPanel.updateAnalyzer(this.analyzer);
    aTestConfigurationTable.updateAnalyzer(this.analyzer);

    WorklistRunnable wr = Common.hm.get(analyzer.getId());
    if (wr != null)
    {
      btnStartResetWorklist.setText("Running");
      btnStartResetWorklist.setEnabled(false);
      btnStartResetWorklist.setIcon(AImageUtil.processing);
      btnRemoveTest.setEnabled(false);
      btnAddTest.setEnabled(false);
    }
    else if (analyzer.allWorklistTestFinished())
    {
      btnStartResetWorklist.setText("Reset");
      btnStartResetWorklist.setEnabled(true);
      btnStartResetWorklist.setIcon(null);
      btnRemoveTest.setEnabled(true);
      btnAddTest.setEnabled(true);

    }
    else
    {
      btnStartResetWorklist.setText("Start");
      btnStartResetWorklist.setEnabled(true);
      btnStartResetWorklist.setIcon(null);
      btnRemoveTest.setEnabled(true);
      btnAddTest.setEnabled(true);
    }

    this.updateUI();
  }

  private void createTabs()
  {

    MyActionListener myActionListener = new MyActionListener();

    tabsPanel = new JPanel(new BorderLayout(0, 0));
    tabsLabelPanel = new JPanel(new GridLayout(0, 3, 2, 0));
    cardPanel = new JPanel();
    cardPanel.setOpaque(true);
    cardPanel.setBackground(Color.WHITE);
    centerButtonsPanel = new JPanel();
    centerButtonsPanel.setOpaque(true);
    centerButtonsPanel.setBackground(Color.WHITE);

    westButtonsPanel = new JPanel();
    westButtonsPanel.setOpaque(true);
    westButtonsPanel.setBackground(Color.WHITE);

    fullButtonsPanel = new JPanel(new BorderLayout());
    fullButtonsPanel.setOpaque(true);
    fullButtonsPanel.setBackground(Color.WHITE);

    cardPanel.setLayout(cardLayout);
    centerButtonsPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    westButtonsPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

    tabWorklist = new MyTabLabel(strWorklist);
    tabNotifications = new MyTabLabel(strNotifications);
    tabTestConfiguration = new MyTabLabel(strTestConfiguration);

    tabWorklist.addActionListener(myActionListener);
    tabNotifications.addActionListener(myActionListener);
    tabTestConfiguration.addActionListener(myActionListener);

    tabWorklist.setSelected(true);

    tabsLabelPanel.add(tabWorklist);
    tabsLabelPanel.add(tabNotifications);
    tabsLabelPanel.add(tabTestConfiguration);
    tabsLabelPanel.setPreferredSize(new Dimension(300, 52));

    btnAddTest = new AButton("Add Test");
    btnRemoveTest = new AButton("Remove Selected Test");
    btnCreateTest = new AButton("Create Test");
    btnStartResetWorklist = new AButton("Start");

    btnRemoveTest.addActionListener(new ActionListener()
    {

      @Override
      public void actionPerformed(ActionEvent e)
      {
        if (aWorklistTable.getSelectedRow() != -1)
        {
          int response = JOptionPane.showConfirmDialog(AnalyzerSimulatorWindow.mainFrame, "Do you want to remove the selected test?", "Question", JOptionPane.YES_NO_OPTION);
          if (response == JOptionPane.OK_OPTION)
          {
            WorklistTestData aWLTData = analyzer.getWorklist().get(0).getWorklistTestList().get(aWorklistTable.getSelectedRow());
            WorklistDAO aWDAO = new WorklistDAO();
            if (aWDAO.deleteTestFromWorklist(aWLTData.getId(), aWLTData.getAnalyzerID(), aWLTData.getWorklistID()))
            {
              analyzer.getWorklist().get(0).getWorklistTestList().remove(aWorklistTable.getSelectedRow());
              aWorklistTable.updateTableData();
            }
          }
        }
      }

    });

    btnAddTest.addActionListener(new ActionListener()
    {

      @Override
      public void actionPerformed(ActionEvent e)
      {
        AddTestWorklistDialog dialog = new AddTestWorklistDialog(analyzer);
        dialog.setLocationRelativeTo(AnalyzerSimulatorWindow.mainFrame);
        dialog.pack();
        dialog.setResizable(false);
        dialog.setVisible(true);
        aWorklistTable.updateTableData();
      }

    });

    btnCreateTest.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        CreateTestDialog dialog = new CreateTestDialog(analyzer);
        dialog.setLocationRelativeTo(AnalyzerSimulatorWindow.mainFrame);
        dialog.pack();
        dialog.setResizable(false);
        dialog.setVisible(true);
      }
    });

    WorklistRunnable wr = Common.hm.get(analyzer.getId());
    if (wr != null)
    {
      btnStartResetWorklist.setText("Running");
      btnStartResetWorklist.setEnabled(false);
      btnStartResetWorklist.setIcon(AImageUtil.processing);
      btnRemoveTest.setEnabled(false);
      btnAddTest.setEnabled(false);
    }
    else if (analyzer.allWorklistTestFinished())
    {
      btnStartResetWorklist.setText("Reset");
      btnRemoveTest.setEnabled(true);
      btnAddTest.setEnabled(true);
    }

    btnStartResetWorklist.addActionListener(new ActionListener()
    {

      @Override
      public void actionPerformed(ActionEvent e)
      {
        if (((AButton) e.getSource()).getText().equals("Start"))
        {
          WorklistRunnable wR = new WorklistRunnable(AAnalyzerPanel.this);
          Common.hm.put(analyzer.getId(), wR);
          wR.start();
          btnStartResetWorklist.setText("Running");
          btnStartResetWorklist.setEnabled(false);
          btnStartResetWorklist.setIcon(AImageUtil.processing);
          btnStartResetWorklist.updateUI();
          btnRemoveTest.setEnabled(false);
          btnAddTest.setEnabled(false);
        }
        else
        {
          WorklistDAO wDAO = new WorklistDAO();
          wDAO.resetWorklist(analyzer.getId(), analyzer.getWorklist().get(0).getId());
          analyzer.getWorklist().get(0).resetWorklistTests();
          aWorklistTable.updateTableData();
          btnStartResetWorklist.setText("Start");
          btnStartResetWorklist.setEnabled(true);
          btnStartResetWorklist.setIcon(null);
          btnStartResetWorklist.updateUI();
          btnRemoveTest.setEnabled(true);
          btnAddTest.setEnabled(true);
        }
      }

    });

    centerButtonsPanel.add(btnRemoveTest);
    centerButtonsPanel.add(btnAddTest);
    centerButtonsPanel.add(btnCreateTest);

    westButtonsPanel.add(btnStartResetWorklist);

    btnStartResetWorklist.setEnabled(true);

    btnCreateTest.setVisible(false);

    fullButtonsPanel.add(westButtonsPanel, BorderLayout.WEST);
    fullButtonsPanel.add(centerButtonsPanel, BorderLayout.CENTER);

    aWorklistTable = new WorklistTable(analyzer);
    aNotificationsPanel = new NotificationsPanel(analyzer, aAnalyzerDetailLabel);
    aTestConfigurationTable = new TestConfigurationTable(analyzer);

    cardPanel.add(new JScrollPane(aWorklistTable), strWorklist);
    cardPanel.add(aNotificationsPanel, strNotifications);
    cardPanel.add(new JScrollPane(aTestConfigurationTable), strTestConfiguration);

    tabsPanel.add(tabsLabelPanel, BorderLayout.NORTH);
    tabsPanel.add(cardPanel, BorderLayout.CENTER);
    tabsPanel.add(fullButtonsPanel, BorderLayout.SOUTH);
  }

  public AnalyzerData getAnalyzer()
  {
    return analyzer;
  }

  class MyTabLabel extends JButton
  {

    private boolean isSelected = false;

    public MyTabLabel(String pTitle)
    {
      super(pTitle);
      this.setOpaque(true);
      this.setBackground(Color.WHITE);
      this.setBorderPainted(false);
      this.setFocusable(false);
      this.setHorizontalTextPosition(SwingConstants.CENTER);
      this.setVerticalTextPosition(SwingConstants.CENTER);
      this.setName(pTitle);
    }

    public void setSelected(boolean isSelected)
    {
      this.isSelected = isSelected;
    }

    @Override
    public void paint(Graphics g)
    {

      Graphics2D g2d = (Graphics2D) g;
      RenderingHints oldRenderingHints = g2d.getRenderingHints();

      super.paint(g2d);

      g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

      g.setColor(this.isSelected ? UIUtils.lineTabSelectedColor : this.getBackground());
      g.fillRect(0, this.getHeight() - 10, this.getWidth(), this.getHeight() - 10);

      g2d.setRenderingHints(oldRenderingHints);
    }

    @Override
    public int getHeight()
    {
      return 50;
    }

  }

  private void setButtonsVisible(boolean pEnable, boolean pAlmostOne)
  {
    btnStartResetWorklist.setVisible(pEnable && pAlmostOne);
    btnAddTest.setVisible(pEnable && pAlmostOne);
    btnRemoveTest.setVisible(pEnable && pAlmostOne);
    btnCreateTest.setVisible(!pEnable && pAlmostOne);
  }

  public AButton getBtnStartResetWorklist()
  {
    return btnStartResetWorklist;
  }

  public AButton getBtnAddTest()
  {
    return btnAddTest;
  }

  public AButton getBtnRemoveTest()
  {
    return btnRemoveTest;
  }

  public WorklistTable getaWorklistTable()
  {
    return aWorklistTable;
  }

  class MyActionListener implements ActionListener
  {

    @Override
    public void actionPerformed(ActionEvent e)
    {
      MyTabLabel aLabel = (MyTabLabel) e.getSource();
      System.out.println(aLabel.getName());

      if (aLabel.getName().equals(strWorklist))
      {
        tabWorklist.setSelected(true);
        tabNotifications.setSelected(false);
        tabTestConfiguration.setSelected(false);
        cardLayout.show(cardPanel, strWorklist);
        setButtonsVisible(true, true);
      }
      else if (aLabel.getName().equals(strNotifications))
      {
        tabWorklist.setSelected(false);
        tabNotifications.setSelected(true);
        tabTestConfiguration.setSelected(false);
        cardLayout.show(cardPanel, strNotifications);
        setButtonsVisible(false, false);
      }
      else
      {
        tabWorklist.setSelected(false);
        tabNotifications.setSelected(false);
        tabTestConfiguration.setSelected(true);
        cardLayout.show(cardPanel, strTestConfiguration);
        setButtonsVisible(false, true);
      }
      tabsLabelPanel.updateUI();
    }
  }
}
