/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.components;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;

import javax.swing.JButton;

import com.tfm_uoc.common.UIUtils;

/**
 * @project TFM - Analyzer Simulator
 * @author Ruben Carmona
 * @filename AButton.java
 */
public class AButton extends JButton
{

  public AButton(String pButtonText)
  {
    super(pButtonText);
    initButton();
  }

  public AButton()
  {
    initButton();
  }

  private void initButton()
  {
    this.setBorderPainted(false);
    this.setContentAreaFilled(false);
    this.setForeground(UIUtils.enabledForeground);
    this.setBackground(UIUtils.enabledBackground);
    this.setMargin(new Insets(5, 10, 5, 10));
    this.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseEntered(java.awt.event.MouseEvent evt)
      {
        if (AButton.this.isEnabled())
        {
          AButton.this.setBackground(UIUtils.rolloverBackground);
          AButton.this.setForeground(UIUtils.enabledBackground);
        }
        else
        {
          AButton.this.setBackground(UIUtils.disabledBackground);
          AButton.this.setForeground(UIUtils.disabledForeground);
        }
      }

      public void mouseExited(java.awt.event.MouseEvent evt)
      {
        AButton.this.setBackground(AButton.this.isEnabled() ? UIUtils.enabledBackground : UIUtils.disabledBackground);
        AButton.this.setForeground(AButton.this.isEnabled() ? UIUtils.enabledForeground : UIUtils.disabledForeground);
      }
    });
  }

  @Override
  public void setEnabled(boolean pEnable)
  {
    super.setEnabled(pEnable);
    this.setBackground(pEnable ? UIUtils.enabledBackground : UIUtils.disabledBackground);
    this.setForeground(pEnable ? UIUtils.enabledForeground : UIUtils.disabledForeground);
  }

  @Override
  public void paint(Graphics g)
  {
    Graphics2D g2d = (Graphics2D) g;
    RenderingHints oldRenderingHints = g2d.getRenderingHints();

    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

    Dimension arcs = new Dimension(8, 8);
    int width = getWidth();
    int height = getHeight();

    if (this.isEnabled())
    {
      g2d.setColor(this.getBackground());
    }
    else
    {
      g2d.setColor(UIUtils.disabledBackground);
    }

    g2d.fillRoundRect(0, 0, this.getWidth(), this.getHeight(), 8, 8);
    g2d.setColor(getBackground());
    g2d.fillRoundRect(0, 0, width - 1, height - 1, arcs.width, arcs.height);// paint background
    g2d.setColor(getForeground());
    g2d.drawRoundRect(0, 0, width - 1, height - 1, arcs.width, arcs.height);// paint border

    super.paint(g2d);

    g2d.setRenderingHints(oldRenderingHints);
  }

}
