/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.SimpleDateFormat;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import com.tfm_uoc.common.AImageUtil;
import com.tfm_uoc.common.PushNotifications;
import com.tfm_uoc.common.UIUtils;
import com.tfm_uoc.database.NotificationsDAO;
import com.tfm_uoc.entities.AnalyzerData;
import com.tfm_uoc.entities.NotificationData;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename NotificationsPanel.java
 */
public class NotificationsPanel extends JPanel
{
  private final SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy a '-' HH:mm:ss");
  private final String[] columnNames = { "Kind", "Notification Description", "When" };

  private MyTableCellRenderer myTableCellRenderer = new MyTableCellRenderer();
  private JPanel pane;
  private JTable aTable;
  private AnalyzerData analyzer;
  private NotificationsComboBox cmbAlarms, cmbWarnings, cmbTestsFeasibility;
  private NotificationsTableModel ntm = new NotificationsTableModel();
  private AButton btnAlarm, btnWarning, btnFeasibility;
  private NotificationsDAO notifDAO = new NotificationsDAO();
  private AAnalyzerDetailLabel aAnalyzerDetailLabel;

  public NotificationsPanel(AnalyzerData pAnalyzer, AAnalyzerDetailLabel pAnalyzerDetailLabel)
  {
    init(pAnalyzer);
    this.aAnalyzerDetailLabel = pAnalyzerDetailLabel;
  }

  private void init(AnalyzerData pAnalyzer)
  {
    this.setLayout(new BorderLayout());
    this.setBackground(Color.WHITE);
    pane = new JPanel(new GridBagLayout());
    pane.setBackground(Color.WHITE);
    pane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    cmbAlarms = new NotificationsComboBox(NotificationData.KIND_ALARM);
    cmbWarnings = new NotificationsComboBox(NotificationData.KIND_WARNING);
    cmbTestsFeasibility = new NotificationsComboBox(NotificationData.KIND_TESTFEASIBILITY_INFO);

    btnAlarm = new AButton("Raise Alarm");
    btnWarning = new AButton("Raise Warning");
    btnFeasibility = new AButton("Raise Test Feasibility");

    configurePanel();
    updateAnalyzer(pAnalyzer);
    initTable();
    setBordersAndSizes();
    addActionListeners();

    this.add(pane, BorderLayout.NORTH);
    this.add(new JScrollPane(aTable), BorderLayout.CENTER);
  }

  private void configurePanel()
  {
    GridBagConstraints c = new GridBagConstraints();

    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridx = 0;
    c.gridy = 0;
    pane.add(new JLabel("Alarms"), c);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridx = 0;
    c.gridy = 1;
    pane.add(cmbAlarms, c);

    c.fill = GridBagConstraints.VERTICAL;
    c.weightx = 0.5;
    c.gridx = 1;
    c.gridy = 1;
    pane.add(btnAlarm, c);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 0.5;
    c.gridx = 2;
    c.gridy = 1;
    pane.add(new JLabel("Test Feasibility"), c);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridx = 0;
    c.gridy = 2;
    pane.add(new JLabel("Warnings"), c);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 0.5;
    c.gridx = 2;
    c.gridy = 2;
    pane.add(cmbTestsFeasibility, c);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridx = 0;
    c.gridy = 3;
    pane.add(cmbWarnings, c);

    c.fill = GridBagConstraints.VERTICAL;
    c.weightx = 0.5;
    c.gridx = 1;
    c.gridy = 3;
    pane.add(btnWarning, c);

    c.fill = GridBagConstraints.VERTICAL;
    c.gridx = 2;
    c.gridy = 3;
    pane.add(btnFeasibility, c);
  }

  private void addActionListeners()
  {
    btnAlarm.addActionListener(new ActionListener()
    {

      @Override
      public void actionPerformed(ActionEvent e)
      {
        addNotification(new NotificationData(analyzer.getCode(), (NotificationData) cmbAlarms.getSelectedItem()));
      }

    });
    btnWarning.addActionListener(new ActionListener()
    {

      @Override
      public void actionPerformed(ActionEvent e)
      {
        addNotification(new NotificationData(analyzer.getCode(), (NotificationData) cmbWarnings.getSelectedItem()));
      }

    });
    btnFeasibility.addActionListener(new ActionListener()
    {

      @Override
      public void actionPerformed(ActionEvent e)
      {
        addNotification(new NotificationData(analyzer.getCode(), (NotificationData) cmbTestsFeasibility.getSelectedItem()));
      }

    });
  }

  private void setBordersAndSizes()
  {
    cmbAlarms.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    cmbWarnings.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    cmbTestsFeasibility.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

    btnAlarm.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    btnFeasibility.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    btnWarning.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

    cmbAlarms.setOpaque(true);
    cmbAlarms.setBackground(Color.WHITE);
    cmbWarnings.setOpaque(true);
    cmbWarnings.setBackground(Color.WHITE);
    cmbTestsFeasibility.setOpaque(true);
    cmbTestsFeasibility.setBackground(Color.WHITE);

    btnAlarm.setOpaque(false);
    btnFeasibility.setOpaque(false);
    btnWarning.setOpaque(false);

  }

  private void initTable()
  {
    aTable = new JTable();
    myTableCellRenderer = new MyTableCellRenderer();
    ntm = new NotificationsTableModel();

    aTable.setModel(ntm);

    aTable.setAutoResizeMode(JTable.AUTO_RESIZE_NEXT_COLUMN);
    aTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    aTable.getTableHeader().setReorderingAllowed(false);
    aTable.getTableHeader().setResizingAllowed(false);
    aTable.getTableHeader().setPreferredSize(new Dimension(-1, 30));
    aTable.setShowHorizontalLines(true);
    aTable.setShowVerticalLines(true);
    aTable.setBackground(UIUtils.nonSelectedBackground);

    UIUtils.setTableColumnFixedSize(aTable.getColumnModel().getColumn(0), 40);
    UIUtils.setTableColumnFixedSize(aTable.getColumnModel().getColumn(2), 200);

    aTable.setRowHeight(35);
    for (int i = 0; i < columnNames.length; i++)
    {
      aTable.getColumnModel().getColumn(i).setCellRenderer(myTableCellRenderer);
    }
  }

  public void updateAnalyzer(AnalyzerData pAnalyzer)
  {
    this.analyzer = pAnalyzer;

    cmbTestsFeasibility.updateByAnalyzerTest(this.analyzer.getConfiguredTestsCodes());

    if (ntm != null)
    {
      ntm.fireTableDataChanged();
    }
  }

  private void addNotification(NotificationData aNotifData)
  {
    aNotifData.setAnalyzerID(analyzer.getId());
    System.out.println(aNotifData.toString());
    notifDAO.createAnalyzerNotification(aNotifData);
    analyzer.getNotifications().add(0, aNotifData);
    analyzer.updateNotificationCounters();
    aAnalyzerDetailLabel.updateUI();
    if (ntm != null)
    {
      ntm.fireTableDataChanged();
    }
    PushNotifications.sendNotification(aNotifData);
  }

  private class NotificationsTableModel extends DefaultTableModel
  {
    public Class<?> getColumnClass(int column)
    {
      if (getValueAt(0, column) != null)
        return getValueAt(0, column).getClass();
      return null;
    }

    public boolean isCellEditable(int row, int column)
    {
      return false;
    }

    public String getColumnName(int column)
    {
      return columnNames[column];
    }

    public int getRowCount()
    {
      if (analyzer != null)
      {
        return analyzer.getNotifications().size();
      }
      else
      {
        return 0;
      }
    }

    public int getColumnCount()
    {
      return columnNames.length;
    }

    public Object getValueAt(int row, int column)
    {
      NotificationData aNotification = analyzer.getNotifications().get(row);

      switch (column)
      {
        case 0:
          JLabel aLabel = new JLabel();
          aLabel.setOpaque(false);
          aLabel.setHorizontalAlignment(JLabel.CENTER);
          aLabel.setVerticalTextPosition(JLabel.BOTTOM);
          if (NotificationData.KIND_WARNING.equals(aNotification.getKind()))
          {
            aLabel.setIcon(AImageUtil.warningIcon);
          }
          else if (NotificationData.KIND_ALARM.equals(aNotification.getKind()))
          {
            aLabel.setIcon(AImageUtil.alarmIcon);
          }
          else if (NotificationData.KIND_TESTFEASIBILITY_INFO.equals(aNotification.getKind()))
          {
            aLabel.setIcon(AImageUtil.infoIcon);

          }
          return aLabel;

        case 1:
          return aNotification.getDescription();
        case 2:
          return sdf.format(aNotification.getTime());
      }
      return "";
    }
  }

  private class MyTableCellRenderer implements TableCellRenderer
  {
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
    {
      JLabel aLabel;
      if (value instanceof String)
      {
        aLabel = new JLabel((String) value);
        aLabel.setOpaque(false);
        aLabel.setHorizontalTextPosition(column == 1 ? JLabel.LEFT : JLabel.CENTER);
        aLabel.setHorizontalAlignment(column == 1 ? JLabel.LEFT : JLabel.CENTER);
      }
      else
      {
        aLabel = (JLabel) value;
      }

      aLabel.setOpaque(true);
      aLabel.setBackground(row % 2 == 0 ? UIUtils.nonSelectedBackground : Color.WHITE);
      return aLabel;
    }
  }
}
