/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import com.tfm_uoc.common.AImageUtil;
import com.tfm_uoc.common.UIUtils;
import com.tfm_uoc.entities.AnalyzerData;
import com.tfm_uoc.entities.WorklistTestData;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename WorklistTable.java
 */
public class WorklistTable extends JTable
{

  private AnalyzerData analyzer;
  private WorklistTableModel wtm = new WorklistTableModel();
  private final String[] columnNames = { "", "Code", "Test Description", "Feasibility", "Init Time", "Finished", "Remaning" };

  private final DateFormat aTimeDateFormat = new SimpleDateFormat("H:mm:ss");

  public WorklistTable(AnalyzerData pAnalyzer)
  {
    this.analyzer = pAnalyzer;
    this.setModel(wtm);
    this.setAutoResizeMode(JTable.AUTO_RESIZE_NEXT_COLUMN);
    this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.setEnabled(true);
    this.getTableHeader().setReorderingAllowed(false);
    this.getTableHeader().setResizingAllowed(false);
    this.getTableHeader().setPreferredSize(new Dimension(-1, 30));
    this.setShowHorizontalLines(true);
    this.setShowVerticalLines(true);
    this.setBackground(UIUtils.nonSelectedBackground);
    this.setBorder(BorderFactory.createEmptyBorder());

    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(0), 40);
    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(1), 100);
    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(3), 80);
    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(4), 100);
    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(5), 100);
    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(6), 100);

    this.setRowHeight(35);
    for (int i = 0; i < columnNames.length; i++)
    {
      this.getColumnModel().getColumn(i).setCellRenderer(new MyTableCellRenderer());
    }

  }

  public void updateAnalyzer(AnalyzerData pAnalyzer)
  {
    this.analyzer = pAnalyzer;
    wtm.fireTableDataChanged();
  }

  public void updateTableData()
  {
    wtm.fireTableDataChanged();
  }

  private class WorklistTableModel extends DefaultTableModel
  {
    public Class<?> getColumnClass(int column)
    {
      if (getValueAt(0, column) != null)
        return getValueAt(0, column).getClass();
      return null;
    }

    public boolean isCellEditable(int row, int column)
    {
      return false;
    }

    public String getColumnName(int column)
    {
      return columnNames[column];
    }

    public int getRowCount()
    {
      if (analyzer != null && analyzer.getWorklist() != null && analyzer.getWorklist().size() > 0)
      {
        return analyzer.getWorklist().get(0).getWorklistTestList().size();
      }
      else
      {
        return 0;
      }
    }

    public int getColumnCount()
    {
      return columnNames.length;
    }

    public Object getValueAt(int row, int column)
    {
      WorklistTestData aTest = analyzer.getWorklist().get(0).getWorklistTestList().get(row);

      switch (column)
      {
        case 0:
          JLabel aLabel = new JLabel();
          aLabel.setOpaque(false);
          aLabel.setHorizontalAlignment(JLabel.CENTER);
          aLabel.setVerticalTextPosition(JLabel.BOTTOM);
          if (WorklistTestData.STATUS_OK.equals(aTest.getStatus()))
          {
            aLabel.setIcon(AImageUtil.okIcon);
          }
          else if (WorklistTestData.STATUS_KO.equals(aTest.getStatus()))
          {
            aLabel.setIcon(AImageUtil.errorIcon);
          }
          else if (aTest.getFinishTime() == null)
          {
            if (row > 0 && analyzer.getWorklist().get(0).getWorklistTestList().get(row - 1).getFinishTime() != null)
            {
              aLabel.setIcon(AImageUtil.tubeIcon);
            }
            else if (row == 0)
            {
              aLabel.setIcon(AImageUtil.tubeIcon);
            }
          }
          return aLabel;

        case 1:
          return aTest.getCode();
        case 2:
          return aTest.getDescription();
        case 3:
          return "" + aTest.getFeasibility();
        case 4:
          if (aTest.getFinishTime() == null)
          {
            if (row > 0 && analyzer.getWorklist().get(0).getWorklistTestList().get(row - 1).getFinishTime() != null)
            {
              return aTimeDateFormat.format(aTest.getInitTime());
            }
            else if (row == 0)
            {
              return aTimeDateFormat.format(aTest.getInitTime());
            }
          }
          else
          {
            return aTimeDateFormat.format(aTest.getInitTime());
          }
        case 5:
          return aTest.getFinishTime() != null ? aTimeDateFormat.format(aTest.getFinishTime()) : "--";
        case 6:
          if (aTest.getFinishTime() != null)
          {
            return "";
          }
          else
          {
            if (row > 0 && analyzer.getWorklist().get(0).getWorklistTestList().get(row - 1).getFinishTime() != null)
            {
              return aTest.getMinutes() + " min";
            }
            else if (row == 0)
            {
              return aTest.getMinutes() + " min";
            }
            else
            {
              return "--";
            }
          }
      }
      return "";
    }
  }

  private class MyTableCellRenderer implements TableCellRenderer
  {
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
    {
      JLabel aLabel = new JLabel();
      if (value instanceof String)
      {
        aLabel = new JLabel((String) value);
        aLabel.setOpaque(false);
        aLabel.setHorizontalTextPosition(column == 2 ? JLabel.LEFT : JLabel.CENTER);
        aLabel.setHorizontalAlignment(column == 2 ? JLabel.LEFT : JLabel.CENTER);
      }
      else
      {
        aLabel = (JLabel) value;
      }

      aLabel.setOpaque(true);
      if (isSelected)
      {
        aLabel.setBackground(UIUtils.rolloverBackground);
        aLabel.setForeground(UIUtils.enabledBackground);
      }
      else
      {
        aLabel.setBackground(row % 2 == 0 ? UIUtils.nonSelectedBackground : Color.WHITE);
      }
      return aLabel;
    }
  }
}
