/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.database;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import com.tfm_uoc.entities.AnalyzerData;
import com.tfm_uoc.main.AnalyzerSimulatorWindow;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename AnalyzerDAO.java
 */
public class AnalyzersDAO extends MySQLDAO
{

  public AnalyzersDAO()
  {

  }

  public ArrayList<AnalyzerData> getAnalyzerList()
  {
    ArrayList<AnalyzerData> aAnalyzerList = new ArrayList<>();

    try
    {
      initConnection(AnalyzerSimulatorWindow.serverIP);
      stmt = connect.createStatement();
      resultSet = stmt.executeQuery("SELECT * FROM analyzer_simulator.Analyzer");

      while (resultSet.next())
      {
        AnalyzerData aAnalyzer = new AnalyzerData();
        aAnalyzer.fromResultSet(resultSet);

        String strSQL = "SELECT n.id, n.description, n.kind, an.date, an.Analyzer_id ";
        strSQL += "FROM Notifications n, AnalyzerNotifications an ";
        strSQL += "WHERE n.id = an.Notifications_id AND an.Analyzer_id = ? ORDER BY an.date DESC";

        preparedStmt = connect.prepareStatement(strSQL);
        preparedStmt.setInt(1, aAnalyzer.getId().intValue());

        ResultSet tmpResultSet = preparedStmt.executeQuery();
        aAnalyzer.notificationsFromResultSet(tmpResultSet);

        strSQL = "SELECT at.*, t.* FROM AnalyzerTests at, Tests t WHERE at.Analyzer_id = ? AND at.Tests_id = t.id";

        preparedStmt = connect.prepareStatement(strSQL);
        preparedStmt.setInt(1, aAnalyzer.getId().intValue());

        tmpResultSet = preparedStmt.executeQuery();

        aAnalyzer.configuredTestFromResultSet(tmpResultSet);

        strSQL = "SELECT w.id, w.Analyzer_id, w.description, w.inittime AS winittime, w.finishtime AS wfinishtime ";
        strSQL += "FROM Worklist w WHERE w.Analyzer_id = ? ORDER BY w.inittime DESC";

        preparedStmt = connect.prepareStatement(strSQL);
        preparedStmt.setInt(1, aAnalyzer.getId().intValue());

        tmpResultSet = preparedStmt.executeQuery();

        aAnalyzer.worklistsFromResultSet(tmpResultSet);

        strSQL = "SELECT wa.*, t.*, a.feasibility FROM Worklist w, WorklistAnalyzerTests wa, Tests t, AnalyzerTests a ";
        strSQL += "WHERE w.Analyzer_id = ? AND w.Analyzer_id = wa.Worklist_Analyzer_id  AND w.id = wa.Worklist_id ";
        strSQL += "AND wa.Tests_id = t.id AND a.Analyzer_id = w.Analyzer_id AND a.Tests_id = t.id ORDER BY wa.inittime ASC";

        preparedStmt = connect.prepareStatement(strSQL);
        preparedStmt.setInt(1, aAnalyzer.getId().intValue());

        tmpResultSet = preparedStmt.executeQuery();

        aAnalyzer.worklistsTestFromResultSet(tmpResultSet);

        aAnalyzerList.add(aAnalyzer);
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      super.close();
    }

    return aAnalyzerList;
  }

  public boolean updateConnectionStatus(Integer pAnalyzerID, boolean pConnected)
  {
    boolean ok = true;
    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {

      preparedStmt = connect.prepareStatement("UPDATE Analyzer SET connected = ? WHERE id = ?");
      int i = 0;
      preparedStmt.setBoolean(++i, pConnected);
      preparedStmt.setInt(++i, pAnalyzerID);

      int res = preparedStmt.executeUpdate();
      ok = res == 1;

    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }
    return ok;
  }

}
