/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.database;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;

import com.tfm_uoc.common.Common;
import com.tfm_uoc.entities.AnalyzerData;
import com.tfm_uoc.entities.NotificationData;
import com.tfm_uoc.main.AnalyzerSimulatorWindow;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename TestDAO.java
 */
public class NotificationsDAO extends MySQLDAO
{

  public ArrayList<NotificationData> getAllAvailableNotificationsByKind(String pKind)
  {
    ArrayList<NotificationData> aNotificationList = new ArrayList<>();

    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      stmt = connect.createStatement();
      preparedStmt = connect.prepareStatement("SELECT * FROM Notifications WHERE kind = ?");
      preparedStmt.setString(1, pKind);
      resultSet = preparedStmt.executeQuery();

      while (resultSet.next())
      {
        NotificationData aNotification = new NotificationData();
        aNotification.fromResultSet(resultSet, false);
        aNotificationList.add(aNotification);
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }

    return aNotificationList;
  }

  public ArrayList<NotificationData> getAllNotificationsForAnAnalyzer(AnalyzerData pAnalyzer)
  {

    ArrayList<NotificationData> aNotificationList = new ArrayList<>();

    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      preparedStmt = connect.prepareStatement("SELECT DISTINCT n.*, an.* FROM Notifications n, AnalyzerNotifications an WHERE an.Analyzer_id = ? AND an.Notifications_id = n.id");
      preparedStmt.setInt(1, pAnalyzer.getId());
      resultSet = preparedStmt.executeQuery();

      while (resultSet.next())
      {
        NotificationData aNotification = new NotificationData();
        aNotification.fromResultSet(resultSet, true);
        aNotificationList.add(aNotification);
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }

    return aNotificationList;

  }

  public String createAnalyzerNotification(NotificationData pNotification)
  {
    String strOk = Common.COMMON_YES;

    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      preparedStmt = connect.prepareStatement("INSERT INTO AnalyzerNotifications VALUES (default, ?, ?, ?)");
      int i = 0;
      preparedStmt.setInt(++i, pNotification.getId());
      preparedStmt.setInt(++i, pNotification.getAnalyzerID());
      preparedStmt.setTimestamp(++i, new Timestamp(System.currentTimeMillis()));
      preparedStmt.executeUpdate();
    }
    catch (Exception e)
    {
      System.err.println(e.getMessage());
      strOk = Common.COMMON_NO + " - " + e.getLocalizedMessage();
    }
    finally
    {
      this.close();
    }

    return strOk;
  }

  public ArrayList<NotificationData> getAllAvailableNotificationsByKindAndTestCode(String pKind, ArrayList<String> pTestCodeList)
  {
    ArrayList<NotificationData> aNotificationList = new ArrayList<>();

    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      stmt = connect.createStatement();
      preparedStmt = connect.prepareStatement("SELECT * FROM Notifications WHERE kind = ?");
      preparedStmt.setString(1, pKind);
      resultSet = preparedStmt.executeQuery();

      while (resultSet.next())
      {
        NotificationData aNotification = new NotificationData();
        aNotification.fromResultSet(resultSet, false);

        for (String strTestCode : pTestCodeList)
        {
          if (aNotification.getDescription().startsWith(strTestCode + " - "))
          {
            aNotificationList.add(aNotification);
            break;
          }
        }
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }

    return aNotificationList;
  }
}
