/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.database;

import java.sql.SQLException;
import java.util.ArrayList;

import com.tfm_uoc.common.Common;
import com.tfm_uoc.entities.AnalyzerData;
import com.tfm_uoc.entities.TestData;
import com.tfm_uoc.main.AnalyzerSimulatorWindow;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename TestDAO.java
 */
public class TestsDAO extends MySQLDAO
{

  public ArrayList<TestData> getAllAvailableTest(Integer pAnalyzerID)
  {
    ArrayList<TestData> aTestList = new ArrayList<>();

    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      preparedStmt = connect.prepareStatement("SELECT t.*, a.feasibility FROM Tests t, AnalyzerTests a WHERE a.Analyzer_id = ? AND a.Tests_id = t.id");
      preparedStmt.setInt(1, pAnalyzerID);
      resultSet = preparedStmt.executeQuery();
      while (resultSet.next())
      {
        TestData aTest = new TestData();
        aTest.fromResultSet(resultSet, true);
        aTestList.add(aTest);
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }

    return aTestList;
  }

  public ArrayList<TestData> getAllConfiguredTestForAnAnalyzer(AnalyzerData pAnalyzer)
  {

    ArrayList<TestData> aTestList = new ArrayList<>();

    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      preparedStmt = connect.prepareStatement("SELECT DISTINCT t. *, ant. * FROM Tests t, AnalyzerTests ant WHERE ant.Analyzer_id = ? AND ant.Tests_id = t.id");
      preparedStmt.setInt(1, pAnalyzer.getId());
      resultSet = preparedStmt.executeQuery();

      while (resultSet.next())
      {
        TestData aTest = new TestData();
        aTest.fromResultSet(resultSet, true);
        aTestList.add(aTest);
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }

    return aTestList;

  }

  public String createTest(TestData pTest, AnalyzerData pAnalyzerData)
  {
    String strOk = Common.COMMON_YES;

    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      preparedStmt = connect.prepareStatement("INSERT INTO Tests VALUES (default, ?, ?, ?, ?, ?)");
      int i = 0;
      preparedStmt.setString(++i, pTest.getCode());
      preparedStmt.setString(++i, pTest.getDescription());
      preparedStmt.setInt(++i, pTest.getMinutes());
      preparedStmt.setString(++i, pTest.getUnits());
      preparedStmt.setInt(++i, pTest.getFeasibilityWarning());
      preparedStmt.executeUpdate();

      preparedStmt = connect.prepareStatement("SELECT DISTINCT t.* FROM Tests t WHERE t.code = ?");
      preparedStmt.setString(1, pTest.getCode());
      resultSet = preparedStmt.executeQuery();

      TestData aTest = new TestData();
      if (resultSet.next())
      {
        aTest.fromResultSet(resultSet, true);
        aTest.setFeasibility(pTest.getFeasibility());
        preparedStmt = connect.prepareStatement("INSERT INTO AnalyzerTests VALUES (?, ?, ?)");
        i = 0;
        preparedStmt.setInt(++i, pAnalyzerData.getId());
        preparedStmt.setInt(++i, aTest.getId());
        preparedStmt.setInt(++i, pTest.getFeasibility());
        preparedStmt.executeUpdate();

        pAnalyzerData.getConfiguredTests().add(aTest);
      }

    }
    catch (Exception e)
    {
      System.err.println(e.getMessage());
      strOk = Common.COMMON_NO + " - " + e.getLocalizedMessage();
    }
    finally
    {
      this.close();
    }

    return strOk;
  }

  public boolean updateTestFeasibilityWarning(Integer pTestID, Integer pFeasibilityWarning)
  {
    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      preparedStmt = connect.prepareStatement("UPDATE Tests SET feasibility_warning = ? WHERE id = ?");
      int i = 0;
      preparedStmt.setInt(++i, pFeasibilityWarning);
      preparedStmt.setInt(++i, pTestID);
      preparedStmt.executeUpdate();
    }
    catch (Exception e)
    {
      System.err.println(e.getMessage());
      return false;
    }
    finally
    {
      this.close();
    }
    return true;
  }

  public boolean updateAnalyzerTestFeasibility(Integer pAnalyzerID, Integer pTestID, Integer pFeasibility)
  {
    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      preparedStmt = connect.prepareStatement("UPDATE AnalyzerTests SET feasibility = ? WHERE Analyzer_id = ? AND Tests_id = ?");
      int i = 0;
      preparedStmt.setInt(++i, pFeasibility);
      preparedStmt.setInt(++i, pAnalyzerID);
      preparedStmt.setInt(++i, pTestID);
      preparedStmt.executeUpdate();
    }
    catch (Exception e)
    {
      System.err.println(e.getMessage());
      return false;
    }
    finally
    {
      this.close();
    }
    return true;
  }
}
