/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.database;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Calendar;

import com.tfm_uoc.entities.TestData;
import com.tfm_uoc.entities.WorklistTestData;
import com.tfm_uoc.main.AnalyzerSimulatorWindow;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename TestDAO.java
 */
public class WorklistDAO extends MySQLDAO
{

  public WorklistTestData addTestToWorklist(TestData pTestData, Integer pAnalyzerID, Integer pWorklistID)
  {
    WorklistTestData aWTD = new WorklistTestData(pTestData);
    Calendar cal = Calendar.getInstance();

    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {
      aWTD.setWorklistID(pWorklistID);
      aWTD.setAnalyzerID(pAnalyzerID);
      aWTD.setFeasibility(pTestData.getFeasibility());
      aWTD.setFeasibilityWarning(pTestData.getFeasibilityWarning());
      aWTD.setInitTime(new Timestamp(System.currentTimeMillis()));

      preparedStmt = connect
          .prepareStatement("SELECT w.*, t.*, a.feasibility FROM WorklistAnalyzerTests w, Tests t, AnalyzerTests a WHERE w.Tests_id = t.id AND t.id = a.Tests_id AND a.Analyzer_id = w.Worklist_Analyzer_id AND w.Worklist_id = ? AND w.Worklist_Analyzer_id = ? ORDER BY w.inittime DESC LIMIT 1");
      int i = 0;
      preparedStmt.setInt(++i, pWorklistID);
      preparedStmt.setInt(++i, pAnalyzerID);

      resultSet = preparedStmt.executeQuery();

      if (resultSet.next())
      {
        WorklistTestData tmpWTD = new WorklistTestData();
        tmpWTD.fromResultSet(resultSet);

        if (tmpWTD.getFinishTime() != null)
        {
          cal.setTimeInMillis(tmpWTD.getFinishTime().getTime());
          cal.add(Calendar.MINUTE, 5);
        }
        else if (tmpWTD.getInitTime() != null)
        {
          cal.setTimeInMillis(tmpWTD.getInitTime().getTime());
          cal.add(Calendar.MINUTE, tmpWTD.getMinutes() + 5);
        }

        aWTD.setInitTime(new Timestamp(cal.getTimeInMillis()));
      }
      else
      {
        preparedStmt.close();
        i = 0;
        preparedStmt = connect.prepareStatement("UPDATE Worklist SET inittime = ? WHERE id = ? AND Analyzer_id = ?");
        preparedStmt.setTimestamp(++i, aWTD.getInitTime());
        preparedStmt.setInt(++i, pWorklistID);
        preparedStmt.setInt(++i, pAnalyzerID);

        preparedStmt.executeUpdate();
      }

      preparedStmt.close();

      preparedStmt = connect.prepareStatement("INSERT INTO WorklistAnalyzerTests VALUES (?, ?, ?, ?, null)");
      i = 0;
      preparedStmt.setInt(++i, aWTD.getWorklistID());
      preparedStmt.setInt(++i, aWTD.getAnalyzerID());
      preparedStmt.setInt(++i, aWTD.getId());
      preparedStmt.setTimestamp(++i, aWTD.getInitTime());

      preparedStmt.executeUpdate();

    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }

    return aWTD;
  }

  public boolean deleteTestFromWorklist(Integer pTestID, Integer pAnalyzerID, Integer pWorklistID)
  {
    boolean ok = true;
    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {

      preparedStmt = connect.prepareStatement("DELETE FROM WorklistAnalyzerTests WHERE Worklist_id = ? AND Worklist_Analyzer_id = ? AND Tests_id = ?");
      int i = 0;
      preparedStmt.setInt(++i, pWorklistID);
      preparedStmt.setInt(++i, pAnalyzerID);
      preparedStmt.setInt(++i, pTestID);

      int res = preparedStmt.executeUpdate();
      ok = res == 1;

    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }
    return ok;
  }

  public boolean updateFinishTime(WorklistTestData wtd)
  {
    boolean ok = true;
    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {

      preparedStmt = connect.prepareStatement("UPDATE WorklistAnalyzerTests SET finishtime = ? WHERE Worklist_id = ? AND Worklist_Analyzer_id = ? AND Tests_id = ?");
      int i = 0;
      preparedStmt.setTimestamp(++i, wtd.getFinishTime());
      preparedStmt.setInt(++i, wtd.getWorklistID());
      preparedStmt.setInt(++i, wtd.getAnalyzerID());
      preparedStmt.setInt(++i, wtd.getId());

      int res = preparedStmt.executeUpdate();
      ok = res == 1;

    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }
    return ok;
  }

  public boolean resetWorklist(Integer pAnalyzerID, Integer pWorklistID)
  {
    boolean ok = true;
    initConnection(AnalyzerSimulatorWindow.serverIP);
    try
    {

      preparedStmt = connect.prepareStatement("UPDATE WorklistAnalyzerTests SET finishtime = NULL WHERE Worklist_id = ? AND Worklist_Analyzer_id = ? ");
      int i = 0;
      preparedStmt.setInt(++i, pWorklistID);
      preparedStmt.setInt(++i, pAnalyzerID);

      int res = preparedStmt.executeUpdate();
      ok = res > 0;

    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      this.close();
    }
    return ok;
  }
}
