/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.entities;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Random;

/**
 * @project TFM - Analyzer Simulator
 * @author Ruben Carmona
 * @filename Analyzer.java
 */

public class AnalyzerData
{

  private Integer id;
  private String code;
  private String description;
  private String ip;
  private String type;
  private Boolean connected;
  private ArrayList<WorklistData> worklistList;
  private ArrayList<TestData> configuredTestsList;
  private ArrayList<NotificationData> notificationList;

  private int alarms = 0;
  private int warnings = 0;
  private int info = 0;

  private static int fakeID = 1;

  public Integer getId()
  {
    return id;
  }

  public void setId(Integer id)
  {
    this.id = id;
  }

  public String getCode()
  {
    return code;
  }

  public void setCode(String code)
  {
    this.code = code;
  }

  public String getDescription()
  {
    return description;
  }

  public void setDescription(String description)
  {
    this.description = description;
  }

  public String getIp()
  {
    return ip;
  }

  public void setIp(String ip)
  {
    this.ip = ip;
  }

  public String getType()
  {
    return type;
  }

  public void setType(String type)
  {
    this.type = type;
  }

  public Boolean getConnected()
  {
    return connected;
  }

  public void setConnected(Boolean connected)
  {
    this.connected = connected;
  }

  public int getAlarms()
  {
    return alarms;
  }

  public void setAlarms(int alarms)
  {
    this.alarms = alarms;
  }

  public int getWarnings()
  {
    return warnings;
  }

  public int getInfo()
  {
    return info;
  }

  public void setInfo(int info)
  {
    this.info = info;
  }

  public void setWarnings(int warnings)
  {
    this.warnings = warnings;
  }

  public ArrayList<WorklistData> getWorklist()
  {
    return worklistList;
  }

  public void setWorklist(ArrayList<WorklistData> worklist)
  {
    this.worklistList = worklist;
  }

  public ArrayList<TestData> getConfiguredTests()
  {
    return configuredTestsList;
  }

  public void setConfiguredTests(ArrayList<TestData> configuredTests)
  {
    this.configuredTestsList = configuredTests;
  }

  public ArrayList<NotificationData> getNotifications()
  {
    return notificationList;
  }

  public void setNotifications(ArrayList<NotificationData> notificationList)
  {
    this.notificationList = notificationList;
  }

  public void fromResultSet(ResultSet pRS)
  {
    try
    {
      this.setId(pRS.getInt("id"));
      this.setCode(pRS.getString("code"));
      this.setDescription(pRS.getString("description"));
      this.setIp(pRS.getString("ip"));
      this.setType(pRS.getString("type"));
      this.setConnected(pRS.getBoolean("connected"));
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
  }

  public void configuredTestFromResultSet(ResultSet pRS)
  {
    configuredTestsList = new ArrayList<>();
    try
    {
      while (pRS.next())
      {
        TestData aTest = new TestData();
        aTest.fromResultSet(pRS, true);

        configuredTestsList.add(aTest);
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
  }

  public void notificationsFromResultSet(ResultSet pRS)
  {
    notificationList = new ArrayList<>();
    try
    {
      while (pRS.next())
      {
        NotificationData aNotification = new NotificationData();
        aNotification.fromResultSet(pRS, true);

        updateNotificationFlags(aNotification.getKind());

        notificationList.add(aNotification);
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
  }

  public void worklistsFromResultSet(ResultSet pRS)
  {
    worklistList = new ArrayList<>();

    try
    {
      while (pRS.next())
      {
        WorklistData aWorklist = new WorklistData();
        aWorklist.fromResultSet(pRS);

        worklistList.add(aWorklist);
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
  }

  public void worklistsTestFromResultSet(ResultSet pRS)
  {

    try
    {
      while (pRS.next())
      {
        WorklistTestData wTD = new WorklistTestData();
        wTD.fromResultSet(pRS);
        // System.out.println(wTD.getCode() + " " + wTD.getInitTime().toString());
        this.worklistList.get(0).getWorklistTestList().add(wTD);
      }
    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
  }

  @Override
  public String toString()
  {
    String str = "Analyzer: " + this.getId() + " - " + this.getCode() + "\n";
    str += this.getDescription() + "\n";
    for (TestData aTest : this.getConfiguredTests())
    {
      str += aTest.toString() + "\n";
    }

    return str;
  }

  private void updateNotificationFlags(String pKind)
  {
    switch (pKind)
    {
      case NotificationData.KIND_ALARM:
        alarms++;
        break;
      case NotificationData.KIND_WARNING:
        warnings++;
        break;
      case NotificationData.KIND_TESTFEASIBILITY_INFO:
        info++;
        break;
    }
  }

  public ArrayList<String> getConfiguredTestsCodes()
  {
    ArrayList<String> configuredTestCodes = new ArrayList<>();
    for (TestData aTestData : configuredTestsList)
    {
      configuredTestCodes.add(aTestData.getCode());
    }
    return configuredTestCodes;
  }

  public void generateFake()
  {
    java.util.Random rnd = new java.util.Random();
    this.setId(new Integer(fakeID));
    this.setCode("Code " + fakeID);
    this.setDescription("Analyzer " + fakeID);
    this.setIp("1.1.1.1");
    this.setType("C");
    this.setConnected(fakeID % 2 == 0);
    this.setAlarms(rnd.nextInt(2));
    this.setWarnings(rnd.nextInt(2));

    this.configuredTestsList = new ArrayList<TestData>();

    int testID = 0;

    while (testID < 10)
    {
      TestData aTest = new TestData();
      aTest.setId(new Integer(testID));
      aTest.setCode("Code " + testID);
      aTest.setDescription("Description " + testID);
      aTest.setMinutes(rnd.nextInt(90));
      aTest.setUnits(rnd.nextInt(3) % 2 == 0 ? "%" : "s");
      aTest.setFeasibility(rnd.nextInt(100));

      this.configuredTestsList.add(aTest);

      testID++;
    }

    WorklistData aWorklist = new WorklistData();
    Timestamp initTime = new Timestamp(System.currentTimeMillis());
    aWorklist.setId(new Integer(1));
    aWorklist.setDescription("Worklist - " + this.getCode());
    aWorklist.setInitTime(initTime);

    Calendar cal = Calendar.getInstance();

    for (TestData aTest : this.getConfiguredTests())
    {
      cal.setTimeInMillis(initTime.getTime());
      cal.add(Calendar.MINUTE, aTest.getMinutes());
      Timestamp finishTime = new Timestamp(cal.getTimeInMillis());

      WorklistTestData aWorklistTest = new WorklistTestData(aTest);
      aWorklistTest.setInitTime(initTime);
      aWorklistTest.setFinishTime(finishTime);
      aWorklistTest.setStatus(rnd.nextInt(5) % 2 == 0 ? WorklistTestData.STATUS_OK : WorklistTestData.STATUS_KO);

      aWorklist.getWorklistTestList().add(aWorklistTest);
      initTime = finishTime;

    }

    this.setWorklist(new ArrayList<WorklistData>());
    this.getWorklist().add(aWorklist);

    ArrayList<NotificationData> aNotificationList = new ArrayList<>();

    int alarmCount, warningCount, testFCount;

    alarmCount = warningCount = testFCount = 0;

    for (int i = 0; i < 10; i++)
    {
      NotificationData notif = new NotificationData();
      notif.setId(new Random().nextInt(100));
      notif.setDescription("Notification " + i);
      notif.setKind(notif.getId() % 2 == 0 ? NotificationData.KIND_TESTFEASIBILITY_INFO : notif.getId() % 3 == 0 ? NotificationData.KIND_ALARM : NotificationData.KIND_WARNING);
      notif.setTime(new Timestamp(System.currentTimeMillis()));
      aNotificationList.add(notif);

      switch (notif.getKind())
      {
        case NotificationData.KIND_ALARM:
          alarmCount++;
          break;
        case NotificationData.KIND_WARNING:
          warningCount++;
          break;
        case NotificationData.KIND_TESTFEASIBILITY_INFO:
          testFCount++;
          break;
      }

      this.setAlarms(alarmCount);
      this.setWarnings(warningCount);
      this.setInfo(testFCount);
    }

    this.setNotifications(aNotificationList);
    fakeID++;
  }

  public void updateNotificationCounters()
  {
    alarms = 0;
    warnings = 0;
    info = 0;

    for (NotificationData notif : notificationList)
    {
      switch (notif.getKind())
      {
        case NotificationData.KIND_ALARM:
          alarms++;
          break;
        case NotificationData.KIND_WARNING:
          warnings++;
          break;
        case NotificationData.KIND_TESTFEASIBILITY_INFO:
          info++;
          break;
      }
    }
  }

  /**
   * @return
   */
  public boolean allWorklistTestFinished()
  {
    for (WorklistTestData wtd : this.getWorklist().get(0).getWorklistTestList())
    {
      if (wtd.getFinishTime() == null)
      {
        return false;
      }
    }
    return true;
  }

}
