/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.entities;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename Worklist.java
 */
public class WorklistData
{

  private Integer id;
  private String description;
  private Timestamp initTime;
  private Timestamp finishTime;
  private ArrayList<WorklistTestData> worklistTestList;

  public WorklistData()
  {
    id = null;
    description = null;
    initTime = null;
    finishTime = null;
    worklistTestList = new ArrayList<WorklistTestData>();
  }

  public Integer getId()
  {
    return id;
  }

  public void setId(Integer id)
  {
    this.id = id;
  }

  public String getDescription()
  {
    return description;
  }

  public void setDescription(String description)
  {
    this.description = description;
  }

  public Timestamp getInitTime()
  {
    return initTime;
  }

  public void setInitTime(Timestamp initTime)
  {
    this.initTime = initTime;
  }

  public Timestamp getFinishTime()
  {
    return finishTime;
  }

  public void setFinishTime(Timestamp finishTime)
  {
    this.finishTime = finishTime;
  }

  public ArrayList<WorklistTestData> getWorklistTestList()
  {
    return worklistTestList;
  }

  public void setTestList(ArrayList<WorklistTestData> worklistTestList)
  {
    this.worklistTestList = worklistTestList;
  }

  public void fromResultSet(ResultSet pRS)
  {
    try
    {
      this.setId(pRS.getInt("id"));
      this.setDescription(pRS.getString("description"));
      this.setInitTime(pRS.getTimestamp("winittime"));
      this.setFinishTime(pRS.getTimestamp("wfinishtime"));

      worklistTestList = new ArrayList<>();

    }
    catch (SQLException e)
    {
      System.err.println(e.getMessage());
    }
  }

  public String toHTMLString()
  {
    String str = "<html><body>" + this.getDescription() + "<br>";

    for (WorklistTestData pWTest : this.getWorklistTestList())
    {
      str += "\t" + pWTest.getCode() + " " + pWTest.getDescription() + " --- " + pWTest.getStatus() + " " + pWTest.getFeasibility() + " " + pWTest.getInitTime() + " " + pWTest.getFinishTime() + "<br>";
    }

    return str;
  }

  public String toString()
  {
    return this.getDescription();
  }

  public void resetWorklistTests()
  {
    for (WorklistTestData pWTest : this.getWorklistTestList())
    {
      pWTest.setFinishTime(null);
    }
  }
}
