/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or partial reproduction of this work it is forbidden by any means or process, including printing, reprography, microfilm, computer processing or any other system, as well as the distribution of copies by rental and lending,
 * without the written permission of the author or the limits authorized by the law of intellectual property.
 *******************************************************************************/
package com.tfm_uoc.main;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.jboss.aerogear.unifiedpush.DefaultPushSender;
import org.jboss.aerogear.unifiedpush.PushSender;
import org.jboss.aerogear.unifiedpush.message.MessageResponseCallback;
import org.jboss.aerogear.unifiedpush.message.UnifiedMessage;

import com.tfm_uoc.common.AImageUtil;
import com.tfm_uoc.common.PushNotifications;
import com.tfm_uoc.components.AAnalyzerDetailLabel;
import com.tfm_uoc.components.AAnalyzerList;
import com.tfm_uoc.components.AAnalyzerPanel;
import com.tfm_uoc.components.AButton;
import com.tfm_uoc.database.AnalyzersDAO;
import com.tfm_uoc.entities.AnalyzerData;

/**
 * @project TFM - Analyzer Simulator
 * @author Ruben Carmona
 * @filename AnalyzerSimulatorWindow.java
 */
public class AnalyzerSimulatorWindow extends JFrame
{

  private PushSender defaultPushSender;
  // final PushSender sender = DefaultPushSender.withRootServerURL("http://"++":8080/ag-push/").pushApplicationId("83c7e059-333b-47b0-a355-3b615f79776b").masterSecret("d8af0203-cb73-437f-9ec8-f0c7d56b47b8").build();
  AButton button;

  JPanel detailAnalyzer;
  AAnalyzerList aAnalyzerList;
  AAnalyzerPanel aAnalyzerPanel;
  public static JFrame mainFrame;
  public static String serverIP = "";

  public AnalyzerSimulatorWindow()
  {

    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setTitle("Analyzer Simulator 1.00");
    setSize(1024, 600);
    this.setIconImage(AImageUtil.loadImage("tubeIcon.png").getImage());
    mainFrame = this;

    try
    {
      Class<?> util = Class.forName("com.apple.eawt.Application");
      Method getApplication = util.getMethod("getApplication", new Class[0]);
      Object application = getApplication.invoke(util);
      Class<?> params[] = new Class[1];
      params[0] = Image.class;
      Method setDockIconImage = util.getMethod("setDockIconImage", params);
      setDockIconImage.invoke(application, this.getIconImage());
    }
    catch (ClassNotFoundException e)
    {
      // log exception
    }
    catch (NoSuchMethodException e)
    {
      // log exception
    }
    catch (IllegalAccessException e)
    {
      // log exception
    }
    catch (IllegalArgumentException e1)
    {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
    catch (InvocationTargetException e1)
    {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }

    String aServerIP = (String) JOptionPane.showInputDialog(this, "Please insert the server IP:\n", "Server IP", JOptionPane.PLAIN_MESSAGE, null, null, "https://52.58.245.92");

    // If a string was returned, say so.
    if ((aServerIP != null) && (aServerIP.length() > 0))
    {
      serverIP = aServerIP.substring(aServerIP.lastIndexOf("/") + 1);
      aServerIP += aServerIP.contains("https") ? ":8443" : ":8080";
      PushNotifications.pushServerURL = PushNotifications.pushServerURL.replaceAll("localhost", aServerIP);

    }
    else
    {
      JOptionPane.showMessageDialog(this, "The server IP address is incorrect", "Server IP Error", JOptionPane.ERROR_MESSAGE);
      System.exit(-1);
    }

    DefaultListModel<AAnalyzerDetailLabel> analyzersListModel = new DefaultListModel<AAnalyzerDetailLabel>();
    detailAnalyzer = new JPanel(new BorderLayout());
    aAnalyzerList = new AAnalyzerList(analyzersListModel);

    AnalyzersDAO aDAO = new AnalyzersDAO();
    ArrayList<AnalyzerData> anList = aDAO.getAnalyzerList();
    for (AnalyzerData aAnalyzer : anList)
    {
      analyzersListModel.addElement(new AAnalyzerDetailLabel(aAnalyzer, true));
    }

    aAnalyzerList.setSelectedIndex(0);
    JScrollPane jp = new JScrollPane(aAnalyzerList);
    jp.setBorder(BorderFactory.createEmptyBorder());

    aAnalyzerPanel = new AAnalyzerPanel(anList.get(0));

    detailAnalyzer.add(aAnalyzerPanel, BorderLayout.CENTER);

    JPanel panel = new JPanel(new BorderLayout());

    panel.add(jp, BorderLayout.WEST);

    panel.add(detailAnalyzer, BorderLayout.CENTER);

    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(panel, BorderLayout.CENTER);

    MouseListener mouseListener = new MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
        if (e.getClickCount() == 1)
        {
          AAnalyzerDetailLabel selectedItem = (AAnalyzerDetailLabel) aAnalyzerList.getSelectedValue();
          aAnalyzerPanel.updateAnalyzer(selectedItem.getAnalyzer());
        }
      }
    };

    aAnalyzerList.addMouseListener(mouseListener);

    initPushSender();
  }

  private void initPushSender()
  {
    defaultPushSender = DefaultPushSender.withRootServerURL(serverIP + ":8080/ag-push").pushApplicationId("83c7e059-333b-47b0-a355-3b615f79776b").masterSecret("d8af0203-cb73-437f-9ec8-f0c7d56b47b8").build();
  }

  public void updateListUI()
  {
    aAnalyzerList.updateUI();
  }

  protected void sendAMessage()
  {

    UnifiedMessage unifiedMessage = UnifiedMessage.withMessage()                 // [2]
        .alert("Hello from Java Sender API!").sound("default").badge("1").config().timeToLive(3600)  // [4]
        .build();

    defaultPushSender.send(unifiedMessage, new MessageResponseCallback()
    {      // [5]

      @Override
      public void onComplete()
      {
        // TODO Auto-generated method stub
        System.out.println("Complete");
      }

    });

  }
}
